# -*- coding: utf-8 -*-
#
# Copyright (C) 2009-2013 Sebastien Helleu <flashcode@flashtux.org>
# Copyright (C) 2010 xt <xt@bash.no>
# Copyright (C) 2010 Aleksey V. Zapparov <ixti@member.fsf.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#
# Jabber/XMPP protocol for WeeChat.
# (this script requires WeeChat 0.3.0 (or newer) and xmpppy library)
#
# For help, see /help jabber
# Happy chat, enjoy :)
#
# History:
# 2013-09-30, Nils Görs <freenode.nils_2>:
#     version 1.6: add support of /secure for passwords and jid
#                : fix stdout/stderr when no JID was set
# 2013-05-14, Billiam <billiamthesecond@gmail.com>:
#     version 1.5: fix unicode encoding error in /jabber buddies
# 2013-05-03, Sebastien Helleu <flashcode@flashtux.org>:
#     version 1.4: add tags in user messages: notify_xxx, no_highlight,
#                  nick_xxx, prefix_nick_xxx, log1
# 2012-05-12, Sebastian Rydberg <sr@rydbergtech.se>:
#     version 1.3: Added support for fetching names from roster
# 2012-04-11, Sebastien Helleu <flashcode@flashtux.org>:
#     version 1.2: fix deletion of server options
# 2012-03-09, Sebastien Helleu <flashcode@flashtux.org>:
#     version 1.1: fix reload of config file
# 2012-01-03, Sebastien Helleu <flashcode@flashtux.org>:
#     version 1.0: changes for future compatibility with Python 3.x
# 2011-12-15, Sebastien Helleu <flashcode@flashtux.org>:
#     version 0.9: fix utf-8 encoding problem on jid
# 2011-03-21, Isaac Raway <isaac.raway@gmail.com>:
#     version 0.8: search chat buffer before opening it
# 2011-02-13, Sebastien Helleu <flashcode@flashtux.org>:
#     version 0.7: use new help format for command arguments
# 2010-11-23, xt
#     version 0.6: change format of sent ping, to match RFC
# 2010-10-05, xt, <xt@bash.no>
#     version 0.5: no highlight for status/presence messages
# 2010-10-01, xt, <xt@bash.no>
#     version 0.4:
#     add kick and invite
# 2010-08-03, Aleksey V. Zapparov <ixti@member.fsf.org>:
#     version 0.3:
#     add /jabber priority [priority]
#     add /jabber status [message]
#     add /jabber presence [online|chat|away|xa|dnd]
# 2010-08-02, Aleksey V. Zapparov <ixti@member.fsf.org>:
#     version 0.2.1:
#     fix prexence is set for current resource instead of sending
#         special presences for all buddies
# 2010-08-02, Aleksey V. Zapparov <ixti@member.fsf.org>:
#     version 0.2:
#     add priority and away_priority of resource
# 2010-08-02, Sebastien Helleu <flashcode@flashtux.org>:
#     version 0.1: first official version
# 2010-08-01, ixti <ixti@member.fsf.org>:
#     fix bug with non-ascii resources
# 2010-06-09, iiijjjiii <iiijjjiii@gmail.com>:
#     add connect server and port options (required for google talk)
#     add private option permitting messages to be displayed in separate
#       chat buffers or in a single server buffer
#     add jid aliases
#     add keepalive ping
# 2010-03-17, xt <xt@bash.no>:
#     add autoreconnect option, autoreconnects on protocol error
# 2010-03-17, xt <xt@bash.no>:
#     add autoconnect option, add new command /jmsg with -server option
# 2009-02-22, Sebastien Helleu <flashcode@flashtux.org>:
#     first version (unofficial)
#

SCRIPT_NAME    = "jabber"
SCRIPT_AUTHOR  = "Sebastien Helleu <flashcode@flashtux.org>"
SCRIPT_VERSION = "1.6"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "Jabber/XMPP protocol for WeeChat"
SCRIPT_COMMAND = SCRIPT_NAME

import re
import warnings

import_ok = True

try:
    import weechat
except:
    print("This script must be run under WeeChat.")
    print("Get WeeChat now at: http://www.weechat.org/")
    import_ok = False

# On import, xmpp may produce warnings about using hashlib instead of
# deprecated sha and md5. Since the code producing those warnings is
# outside this script, catch them and ignore.
original_filters = warnings.filters[:]
warnings.filterwarnings("ignore",category=DeprecationWarning)
try:
    import xmpp
except:
    print("Package python-xmpp (xmpppy) must be installed to use Jabber protocol.")
    print("Get xmpppy with your package manager, or at this URL: http://xmpppy.sourceforge.net/")
    import_ok = False
finally:
    warnings.filters = original_filters

# ==============================[ global vars ]===============================

jabber_servers = []
jabber_server_options = {
    "jid"          : { "type"         : "string",
                       "desc"         : "jabber id (user@server.tld)",
                       "min"          : 0,
                       "max"          : 0,
                       "string_values": "",
                       "default"      : "",
                       "value"        : "",
                       "check_cb"     : "",
                       "change_cb"    : "",
                       "delete_cb"    : "",
                       },
    "priority"     : { "type"         : "integer",
                       "desc"         : "Default resource priority",
                       "min"          : 0,
                       "max"          : 65535,
                       "string_values": "",
                       "default"      : "8",
                       "value"        : "8",
                       "check_cb"     : "",
                       "change_cb"    : "",
                       "delete_cb"    : "",
                       },
    "away_priority": { "type"         : "integer",
                       "desc"         : "Resource priority on away",
                       "min"          : 0,
                       "max"          : 65535,
                       "string_values": "",
                       "default"      : "0",
                       "value"        : "0",
                       "check_cb"     : "",
                       "change_cb"    : "",
                       "delete_cb"    : "",
                       },
    "password"     : { "type"         : "string",
                       "desc"         : "password for jabber id on server",
                       "min"          : 0,
                       "max"          : 0,
                       "string_values": "",
                       "default"      : "",
                       "value"        : "",
                       "check_cb"     : "",
                       "change_cb"    : "",
                       "delete_cb"    : "",
                       },
    "server"       : { "type"         : "string",
                       "desc"         : "connect server host or ip, eg. talk.google.com",
                       "min"          : 0,
                       "max"          : 0,
                       "string_values": "",
                       "default"      : "",
                       "value"        : "",
                       "check_cb"     : "",
                       "change_cb"    : "",
                       "delete_cb"    : "",
                       },
    "port"         : { "type"         : "integer",
                       "desc"         : "connect server port, eg. 5223",
                       "min"          : 0,
                       "max"          : 65535,
                       "string_values": "",
                       "default"      : "5222",
                       "value"        : "5222",
                       "check_cb"     : "",
                       "change_cb"    : "",
                       "delete_cb"    : "",
                       },
    "autoconnect"  : { "type"         : "boolean",
                       "desc"         : "automatically connect to server when script is starting",
                       "min"          : 0,
                       "max"          : 0,
                       "string_values": "",
                       "default"      : "off",
                       "value"        : "off",
                       "check_cb"     : "",
                       "change_cb"    : "",
                       "delete_cb"    : "",
                       },
    "autoreconnect": { "type"         : "boolean",
                       "desc"         : "automatically reconnect to server when disconnected",
                       "min"          : 0,
                       "max"          : 0,
                       "string_values": "",
                       "default"      : "off",
                       "value"        : "off",
                       "check_cb"     : "",
                       "change_cb"    : "",
                       "delete_cb"    : "",
                       },
    "private"      : { "type"         : "boolean",
                       "desc"         : "display messages in separate chat buffers instead of a single server buffer",
                       "min"          : 0,
                       "max"          : 0,
                       "string_values": "",
                       "default"      : "on",
                       "value"        : "on",
                       "check_cb"     : "",
                       "change_cb"    : "",
                       "delete_cb"    : "",
                       },
    "ping_interval": { "type"         : "integer",
                       "desc"         : "Number of seconds between server pings. 0 = disable",
                       "min"          : 0,
                       "max"          : 9999999,
                       "string_values": "",
                       "default"      : "0",
                       "value"        : "0",
                       "check_cb"     : "ping_interval_check_cb",
                       "change_cb"    : "",
                       "delete_cb"    : "",
                       },
    "ping_timeout" : { "type"         : "integer",
                       "desc"         : "Number of seconds to allow ping to respond before timing out",
                       "min"          : 0,
                       "max"          : 9999999,
                       "string_values": "",
                       "default"      : "10",
                       "value"        : "10",
                       "check_cb"     : "ping_timeout_check_cb",
                       "change_cb"    : "",
                       "delete_cb"    : "",
                       },
    }
jabber_config_file = None
jabber_config_section = {}
jabber_config_option = {}
jabber_jid_aliases = {}             # { 'alias1': 'jid1', 'alias2': 'jid2', ... }

# =================================[ config ]=================================

def jabber_config_init():
    """ Initialize config file: create sections and options in memory. """
    global jabber_config_file, jabber_config_section
    jabber_config_file = weechat.config_new("jabber", "jabber_config_reload_cb", "")
    if not jabber_config_file:
        return
    # look
    jabber_config_section["look"] = weechat.config_new_section(
        jabber_config_file, "look", 0, 0, "", "", "", "", "", "", "", "", "", "")
    if not jabber_config_section["look"]:
        weechat.config_free(jabber_config_file)
        return
    jabber_config_option["debug"] = weechat.config_new_option(
        jabber_config_file, jabber_config_section["look"],
        "debug", "boolean", "display debug messages", "", 0, 0,
        "off", "off", 0, "", "", "", "", "", "")
    # color
    jabber_config_section["color"] = weechat.config_new_section(
        jabber_config_file, "color", 0, 0, "", "", "", "", "", "", "", "", "", "")
    if not jabber_config_section["color"]:
        weechat.config_free(jabber_config_file)
        return
    jabber_config_option["message_join"] = weechat.config_new_option(
        jabber_config_file, jabber_config_section["color"],
        "message_join", "color", "color for text in join messages", "", 0, 0,
        "green", "green", 0, "", "", "", "", "", "")
    jabber_config_option["message_quit"] = weechat.config_new_option(
        jabber_config_file, jabber_config_section["color"],
        "message_quit", "color", "color for text in quit messages", "", 0, 0,
        "red", "red", 0, "", "", "", "", "", "")
    # server
    jabber_config_section["server"] = weechat.config_new_section(
        jabber_config_file, "server", 0, 0,
        "jabber_config_server_read_cb", "", "jabber_config_server_write_cb", "",
        "", "", "", "", "", "")
    if not jabber_config_section["server"]:
        weechat.config_free(jabber_config_file)
        return
    jabber_config_section["jid_aliases"] = weechat.config_new_section(
        jabber_config_file, "jid_aliases", 0, 0,
        "jabber_config_jid_aliases_read_cb", "",
        "jabber_config_jid_aliases_write_cb", "",
        "", "", "", "", "", "")
    if not jabber_config_section["jid_aliases"]:
        weechat.config_free(jabber_config_file)
        return

def jabber_config_reload_cb(data, config_file):
    """ Reload config file. """
    return weechat.config_reload(config_file)

def jabber_config_server_read_cb(data, config_file, section, option_name, value):
    """ Read server option in config file. """
    global jabber_servers
    rc = weechat.WEECHAT_CONFIG_OPTION_SET_ERROR
    items = option_name.split(".", 1)
    if len(items) == 2:
        server = jabber_search_server_by_name(items[0])
        if not server:
            server = Server(items[0])
            jabber_servers.append(server)
        if server:
            rc = weechat.config_option_set(server.options[items[1]], value, 1)
    return rc

def jabber_config_server_write_cb(data, config_file, section_name):
    """ Write server section in config file. """
    global jabber_servers
    weechat.config_write_line(config_file, section_name, "")
    for server in jabber_servers:
        for name, option in sorted(server.options.items()):
            weechat.config_write_option(config_file, option)
    return weechat.WEECHAT_RC_OK

def jabber_config_jid_aliases_read_cb(data, config_file, section, option_name, value):
    """ Read jid_aliases option in config file. """
    global jabber_jid_aliases
    jabber_jid_aliases[option_name] = value
    option = weechat.config_new_option(
        config_file, section,
        option_name, "string", "jid alias", "", 0, 0,
        "", value, 0, "", "", "", "", "", "")
    if not option:
        return weechat.WEECHAT_CONFIG_OPTION_SET_ERROR
    return weechat.WEECHAT_CONFIG_OPTION_SET_OK_CHANGED

def jabber_config_jid_aliases_write_cb(data, config_file, section_name):
    """ Write jid_aliases section in config file. """
    global jabber_jid_aliases
    weechat.config_write_line(config_file, section_name, "")
    for alias, jid in sorted(jabber_jid_aliases.items()):
        weechat.config_write_line(config_file, alias, jid)
    return weechat.WEECHAT_RC_OK

def jabber_config_read():
    """ Read jabber config file (jabber.conf). """
    global jabber_config_file
    return weechat.config_read(jabber_config_file)

def jabber_config_write():
    """ Write jabber config file (jabber.conf). """
    global jabber_config_file
    return weechat.config_write(jabber_config_file)

def jabber_debug_enabled():
    """ Return True if debug is enabled. """
    global jabber_config_options
    if weechat.config_boolean(jabber_config_option["debug"]):
        return True
    return False

def jabber_config_color(color):
    """ Return color code for a jabber color option. """
    global jabber_config_option
    if color in jabber_config_option:
        return weechat.color(weechat.config_color(jabber_config_option[color]))
    return ""

def ping_timeout_check_cb(server_name, option, value):
    global jabber_config_file, jabber_config_section
    ping_interval_option = weechat.config_search_option(
        jabber_config_file,
        jabber_config_section["server"],
        "%s.ping_interval" % (server_name)
        )
    ping_interval = weechat.config_integer(ping_interval_option)
    if int(ping_interval) and int(value) >= int(ping_interval):
        weechat.prnt("", "\njabber: unable to update 'ping_timeout' for server %s" % (server_name))
        weechat.prnt("", "jabber: to prevent multiple concurrent pings, ping_interval must be greater than ping_timeout")
        return weechat.WEECHAT_CONFIG_OPTION_SET_ERROR
    return weechat.WEECHAT_CONFIG_OPTION_SET_OK_CHANGED

def ping_interval_check_cb(server_name, option, value):
    global jabber_config_file, jabber_config_section
    ping_timeout_option = weechat.config_search_option(
        jabber_config_file,
        jabber_config_section["server"],
        "%s.ping_timeout" % (server_name)
        )
    ping_timeout = weechat.config_integer(ping_timeout_option)
    if int(value) and int(ping_timeout) >= int(value):
        weechat.prnt("", "\njabber: unable to update 'ping_interval' for server %s" % (server_name))
        weechat.prnt("", "jabber: to prevent multiple concurrent pings, ping_interval must be greater than ping_timeout")
        return weechat.WEECHAT_CONFIG_OPTION_SET_ERROR
    return weechat.WEECHAT_CONFIG_OPTION_SET_OK_CHANGED

# ================================[ servers ]=================================

class Server:
    """ Class to manage a server: buffer, connection, send/recv data. """

    def __init__(self, name, **kwargs):
        """ Init server """
        global jabber_config_file, jabber_config_section, jabber_server_options
        self.name = name
        # create options (user can set them with /set)
        self.options = {}
        # if the value is provided, use it, otherwise use the default
        values = {}
        for option_name, props in jabber_server_options.items():
            values[option_name] = props["default"]
        values['name'] = name
        values.update(**kwargs)
        for option_name, props in jabber_server_options.items():
            self.options[option_name] = weechat.config_new_option(
                jabber_config_file, jabber_config_section["server"],
                self.name + "." + option_name, props["type"], props["desc"],
                props["string_values"], props["min"], props["max"],
                props["default"], values[option_name], 0,
                props["check_cb"], self.name, props["change_cb"], "",
                props["delete_cb"], "")
        # internal data
        self.jid = None
        self.client = None
        self.sock = None
        self.hook_fd = None
        self.buffer = ""
        self.chats = []
        self.roster = None
        self.buddies = []
        self.buddy = None
        self.ping_timer = None              # weechat.hook_timer for sending pings
        self.ping_timeout_timer = None      # weechat.hook_timer for monitoring ping timeout
        self.ping_up = False                # Connection status as per pings.
        self.presence = xmpp.protocol.Presence()

    def option_string(self, option_name):
        """ Return a server option, as string. """
        return weechat.config_string(self.options[option_name])

    def option_boolean(self, option_name):
        """ Return a server option, as boolean. """
        return weechat.config_boolean(self.options[option_name])

    def option_integer(self, option_name):
        """ Return a server option, as string. """
        return weechat.config_integer(self.options[option_name])

    def connect(self):
        """ Connect to Jabber server. """
        if not self.buffer:
            bufname = "%s.server.%s" % (SCRIPT_NAME, self.name)
            self.buffer = weechat.buffer_search("python", bufname)
            if not self.buffer:
                self.buffer = weechat.buffer_new(bufname,
                                                 "jabber_buffer_input_cb", "",
                                                 "jabber_buffer_close_cb", "")
            if self.buffer:
                weechat.buffer_set(self.buffer, "short_name", self.name)
                weechat.buffer_set(self.buffer, "localvar_set_type", "server")
                weechat.buffer_set(self.buffer, "localvar_set_server", self.name)
                weechat.buffer_set(self.buffer, "nicklist", "1")
                weechat.buffer_set(self.buffer, "nicklist_display_groups", "1")
                weechat.buffer_set(self.buffer, "display", "auto")
        self.disconnect()

        if not eval_expression(self.option_string("jid")):
            weechat.prnt(self.buffer, "%sjabber: JID must contain at least domain name"
                         % weechat.prefix("error"))
            self.ping_up = False
            self.client = None
            return self.is_connected()

        self.buddy = Buddy(jid=eval_expression(self.option_string("jid")), server=self)

        server = self.option_string("server")
        port = self.option_integer("port")
        self.client = xmpp.Client(server=self.buddy.domain, debug=[])
        conn = None
        server_tuple = None
        if server:
            if port:
                server_tuple = (server, port)
            else:
                server_tuple = (server)

        # self.client.connect() may produce a "socket.ssl() is deprecated"
        # warning. Since the code producing the warning is outside this script,
        # catch it and ignore.
        original_filters = warnings.filters[:]
        warnings.filterwarnings("ignore",category=DeprecationWarning)
        try:
            conn = self.client.connect(server=server_tuple)
        finally:
            warnings.filters = original_filters

        if conn:
            weechat.prnt(self.buffer, "jabber: connection ok with %s" % conn)
            res = self.buddy.resource
            if not res:
                res = "WeeChat"

            auth = self.client.auth(self.buddy.username,
                                    eval_expression(self.option_string("password")),
                                    res)

            if auth:
                weechat.prnt(self.buffer, "jabber: authentication ok (using %s)" % auth)

                self.roster = self.client.getRoster()
                self.client.RegisterHandler("presence", self.presence_handler)
                self.client.RegisterHandler("iq", self.iq_handler)
                self.client.RegisterHandler("message", self.message_handler)
                self.client.sendInitPresence(requestRoster=1)
                self.sock = self.client.Connection._sock.fileno()
                self.hook_fd = weechat.hook_fd(self.sock, 1, 0, 0, "jabber_fd_cb", "")
                weechat.buffer_set(self.buffer, "highlight_words", self.buddy.username)
                weechat.buffer_set(self.buffer, "localvar_set_nick", self.buddy.username);
                hook_away = weechat.hook_command_run("/away -all*", "jabber_away_command_run_cb", "")


                # setting initial presence
                priority = weechat.config_integer(self.options['priority'])
                self.set_presence(show="",priority=priority)


                self.ping_up = True
            else:
                weechat.prnt(self.buffer, "%sjabber: could not authenticate"
                             % weechat.prefix("error"))
                self.ping_up = False
                self.client = None
        else:
            weechat.prnt(self.buffer, "%sjabber: could not connect"
                         % weechat.prefix("error"))
            self.ping_up = False
            self.client = None
        return self.is_connected()

    def is_connected(self):
        """Return connect status"""
        if not self.client or not self.client.isConnected():
            return False
        else:
            return True

    def add_chat(self, buddy):
        """Create a chat buffer for a buddy"""
        chat = Chat(self, buddy, switch_to_buffer=False)
        self.chats.append(chat)
        return chat

    def add_buddy(self, jid):
        """ Add a new buddy """
        self.client.Roster.Authorize(jid)
        self.client.Roster.Subscribe(jid)

    def del_buddy(self, jid):
        """ Remove a buddy and/or deny authorization request """
        self.client.Roster.Unauthorize(jid)
        self.client.Roster.Unsubscribe(jid)

    def print_debug_server(self, message):
        """ Print debug message on server buffer. """
        if jabber_debug_enabled():
            weechat.prnt(self.buffer, "%sjabber: %s" % (weechat.prefix("network"), message))

    def print_debug_handler(self, handler_name, node):
        """ Print debug message for a handler on server buffer. """
        self.print_debug_server("%s_handler, xml message:\n%s"
                                % (handler_name,
                                   node.__str__(fancy=True).encode("utf-8")))

    def print_error(self, message):
        """ Print error message on server buffer. """
        if jabber_debug_enabled():
            weechat.prnt(self.buffer, "%sjabber: %s" % (weechat.prefix("error"), message))

    def presence_handler(self, conn, node):
        self.print_debug_handler("presence", node)
        buddy = self.search_buddy_list(node.getFrom().getStripped().encode("utf-8"), by='jid')
        if not buddy:
            buddy = self.add_buddy(jid=node.getFrom())
        action='update'
        node_type = node.getType()
        if node_type in ["error", "unavailable"]:
            action='remove'
        if action == 'update':
            away = node.getShow() in ["away", "xa"]
            status = ''
            if node.getStatus():
                status = node.getStatus().encode("utf-8")
            if self.roster:
                name = self.roster.getName(buddy.bare_jid)
                if name:
                    buddy.set_name(name.encode("utf-8"))
            buddy.set_status(status=status, away=away)
        self.update_nicklist(buddy=buddy, action=action)
        return

    def iq_handler(self, conn, node):
        """ Receive iq message. """
        self.print_debug_handler("iq", node)
        #weechat.prnt(self.buffer, "jabber: iq handler")
        if node.getFrom() == self.buddy.domain:
            # type='result' => pong from server
            # type='error'  => error message from server
            # The ping_up is set True on an error message to handle cases where
            # the ping feature is not implemented on a server. It's a bit of a
            # hack, but if we can receive an error from the server, we assume
            # the connection to the server is up.
            if node.getType() in ['result', 'error']:
                self.delete_ping_timeout_timer()    # Disable the timeout feature
                self.ping_up = True
                if not self.client.isConnected() and weechat.config_boolean(self.options['autoreconnect']):
                    self.connect()

    def message_handler(self, conn, node):
        """ Receive message. """
        self.print_debug_handler("message", node)
        node_type = node.getType()
        if node_type not in ["message", "chat", None]:
            self.print_error("unknown message type: '%s'" % node_type)
            return
        jid = node.getFrom()
        body = node.getBody()
        if not jid or not body:
            return
        buddy = self.search_buddy_list(self.stringify_jid(jid), by='jid')
        if not buddy:
            buddy = self.add_buddy(jid=jid)
        # If a chat buffer exists for the buddy, receive the message with that
        # buffer even if private is off. The buffer may have been created with
        # /jchat.
        recv_object = self
        if not buddy.chat and weechat.config_boolean(self.options['private']):
            self.add_chat(buddy)
        if buddy.chat:
            recv_object = buddy.chat
        recv_object.recv_message(buddy, body.encode("utf-8"))

    def recv(self):
        """ Receive something from Jabber server. """
        if not self.client:
            return
        try:
            self.client.Process(1)
        except xmpp.protocol.StreamError as e:
            weechat.prnt('', '%s: Error from server: %s' %(SCRIPT_NAME, e))
            self.disconnect()
            if weechat.config_boolean(self.options['autoreconnect']):
                autoreconnect_delay = 30
                weechat.command('', '/wait %s /%s connect %s' %
                                (autoreconnect_delay, SCRIPT_COMMAND, self.name))

    def recv_message(self, buddy, message):
        """ Receive a message from buddy. """
        weechat.prnt_date_tags(self.buffer, 0,
                               "notify_private,nick_%s,prefix_nick_%s,log1" %
                               (buddy.alias,
                                weechat.config_string(weechat.config_get("weechat.color.chat_nick_other"))),
                               "%s%s\t%s" % (weechat.color("chat_nick_other"),
                                             buddy.alias,
                                             message))

    def print_status(self, nickname, status):
        """ Print a status in server window and in chat. """
        weechat.prnt_date_tags(self.buffer, 0, "no_highlight", "%s%s has status %s" %
                               (weechat.prefix("action"),
                                nickname,
                                status))
        for chat in self.chats:
            if nickname in chat.buddy.alias:
                chat.print_status(status)
                break

    def send_message(self, buddy, message):
        """ Send a message to buddy.

        The buddy argument can be either a jid string,
        eg username@domain.tld/resource or a Buddy object instance.
        """
        recipient = buddy
        if isinstance(buddy, Buddy):
            recipient = buddy.jid
        if not self.ping_up:
            weechat.prnt(self.buffer, "%sjabber: unable to send message, connection is down"
                         % weechat.prefix("error"))
            return
        if self.client:
            msg = xmpp.protocol.Message(to=recipient, body=message, typ='chat')
            self.client.send(msg)

    def send_message_from_input(self, input=''):
        """ Send a message from input text on server buffer. """
        # Input must be of format "name: message" where name is a jid, bare_jid
        # or alias. The colon can be replaced with a comma as well.
        # Split input into name and message.
        if not re.compile(r'.+[:,].+').match(input):
            weechat.prnt(self.buffer, "%sjabber: %s" % (weechat.prefix("network"),
                "Invalid send format. Use  jid: message"
                ))
            return
        name, message = re.split('[:,]', input, maxsplit=1)
        buddy = self.search_buddy_list(name, by='alias')
        if not buddy:
            weechat.prnt(self.buffer,
                    "%sjabber: Invalid jid: %s" % (weechat.prefix("network"),
                    name))
            return
        # Send activity indicates user is no longer away, set it so
        if self.buddy and self.buddy.away:
            self.set_away('')
        self.send_message(buddy=buddy, message=message)
        try:
            sender = self.buddy.alias
        except:
            sender = self.jid
        weechat.prnt_date_tags(self.buffer, 0,
                               "notify_none,no_highlight,nick_%s,prefix_nick_%s,log1" %
                               (sender,
                                weechat.config_string(weechat.config_get("weechat.color.chat_nick_self"))),
                               "%s%s\t%s" % (weechat.color("chat_nick_self"),
                                             sender,
                                             message.strip()))

    def set_away(self, message):
        """ Set/unset away on server.

        If a message is provided, status is set to 'away'.
        If no message, then status is set to 'online'.
        """
        if message:
            show = "xa"
            status = message
            priority = weechat.config_integer(self.options['away_priority'])
            self.buddy.set_status(away=True, status=message)
        else:
            show = ""
            status = None
            priority = weechat.config_integer(self.options['priority'])
            self.buddy.set_status(away=False)
        self.set_presence(show, status, priority)

    def set_presence(self, show=None, status=None, priority=None):
        if not show == None: self.presence.setShow(show)
        if not status == None: self.presence.setStatus(status)
        if not priority == None: self.presence.setPriority(priority)
        self.client.send(self.presence)

    def add_buddy(self, jid=None):
        buddy = Buddy(jid=jid, server=self)
        buddy.resource = buddy.resource.encode("utf-8")
        self.buddies.append(buddy)
        return buddy

    def display_buddies(self):
        """ Display buddies. """
        weechat.prnt(self.buffer, "")
        weechat.prnt(self.buffer, "Buddies:")

        len_max = { 'alias': 5, 'jid': 5 }
        lines = []
        for buddy in sorted(self.buddies, key=lambda x: x.jid.getStripped().encode('utf-8')):
            alias = ''
            if buddy.alias != buddy.bare_jid:
                alias = buddy.alias
            buddy_jid_string = buddy.jid.getStripped().encode('utf-8')
            lines.append( {
                'jid': buddy_jid_string,
                'alias': alias,
                'status': buddy.away_string(),
                })
            if len(alias) > len_max['alias']:
                len_max['alias'] = len(alias)
            if len(buddy_jid_string) > len_max['jid']:
                len_max['jid'] = len(buddy_jid_string)
        prnt_format = "  %s%-" + str(len_max['jid']) + "s %-" + str(len_max['alias']) + "s %s"
        weechat.prnt(self.buffer, prnt_format % ('', 'JID', 'Alias', 'Status'))
        for line in lines:
            weechat.prnt(self.buffer, prnt_format % (weechat.color("chat_nick"),
                                                    line['jid'],
                                                    line['alias'],
                                                    line['status'],
                                                    ))

    def stringify_jid(self, jid, wresource=1):
        """ Serialise JID into string.

        Args:
            jid: xmpp.protocol.JID, JID instance to serialize

        Notes:
            Method is based on original JID.__str__ but with hack to allow
            non-ascii in resource names.
        """
        if jid.node:
            jid_str = jid.node + '@' + jid.domain
        else:
            jid_str = jid.domain
        if wresource and jid.resource:
            # concatenate jid with resource delimiter first and encode them
            # into utf-8, else it will raise UnicodeException becaouse of
            # slash character :((
            return (jid_str + '/').encode("utf-8") + jid.resource.encode("utf-8")
        return jid_str.encode("utf-8")

    def search_buddy_list(self, name, by='jid'):
        """ Search for a buddy by name.

        Args:
            name: string, the buddy name to search, eg the jid or alias
            by: string, either 'alias' or 'jid', determines which Buddy
                property to match on, default 'jid'

        Notes:
            If the 'by' parameter is set to 'jid', the search matches on all
            Buddy object jid properties, followed by all bare_jid properties.
            Once a match is found it is returned.

            If the 'by' parameter is set to 'alias', the search matches on all
            Buddy object alias properties.

            Generally, set the 'by' parameter to 'jid' when the jid is provided
            from a server, for example from a received message. Set 'by' to
            'alias' when the jid is provided by the user.
        """
        if by == 'jid':
            for buddy in self.buddies:
                if self.stringify_jid(buddy.jid) == name:
                    return buddy
            for buddy in self.buddies:
                if buddy.bare_jid == name:
                    return buddy
        else:
            for buddy in self.buddies:
                if buddy.alias == name:
                    return buddy
        return None

    def update_nicklist(self, buddy=None, action=None):
        """Update buddy in nicklist
            Args:
                buddy: Buddy object instance
                action: string, one of 'update' or 'remove'
        """
        if not buddy:
            return
        if not action in ['remove', 'update']:
            return
        ptr_nick_gui = weechat.nicklist_search_nick(self.buffer, "", buddy.alias)
        weechat.nicklist_remove_nick(self.buffer, ptr_nick_gui)
        msg = ''
        prefix = ''
        color = ''
        away = ''
        if action == 'update':
            nick_color = "bar_fg"
            if buddy.away:
                nick_color = "weechat.color.nicklist_away"
            weechat.nicklist_add_nick(self.buffer, "", buddy.alias,
                                      nick_color, "", "", 1)
            if not ptr_nick_gui:
                msg = 'joined'
                prefix = 'join'
                color = 'message_join'
                away = buddy.away_string()
        if action == 'remove':
            msg = 'quit'
            prefix = 'quit'
            color = 'message_quit'
        if msg:
            weechat.prnt(self.buffer, "%s%s%s%s has %s %s"
                         % (weechat.prefix(prefix),
                            weechat.color("chat_nick"),
                            buddy.alias,
                            jabber_config_color(color),
                            msg,
                            away))
        return

    def add_ping_timer(self):
        if self.ping_timer:
            self.delete_ping_timer()
        if not self.option_integer('ping_interval'):
            return
        self.ping_timer = weechat.hook_timer( self.option_integer('ping_interval') * 1000,
                0, 0, "jabber_ping_timer", self.name)
        return

    def delete_ping_timer(self):
        if self.ping_timer:
            weechat.unhook(self.ping_timer)
        self.ping_time = None
        return

    def add_ping_timeout_timer(self):
        if self.ping_timeout_timer:
            self.delete_ping_timeout_timer()
        if not self.option_integer('ping_timeout'):
            return
        self.ping_timeout_timer = weechat.hook_timer(
                self.option_integer('ping_timeout') * 1000, 0, 1,
                "jabber_ping_timeout_timer", self.name)
        return

    def delete_ping_timeout_timer(self):
        if self.ping_timeout_timer:
            weechat.unhook(self.ping_timeout_timer)
        self.ping_timeout_timer = None
        return

    def ping(self):
        if not self.is_connected():
            if not self.connect():
                return
        iq = xmpp.protocol.Iq(to=self.buddy.domain, typ='get')
        iq.addChild( name= "ping", namespace = "urn:xmpp:ping" )
        id = self.client.send(iq)
        self.print_debug_handler("ping", iq)
        self.add_ping_timeout_timer()
        return

    def ping_time_out(self):
        self.delete_ping_timeout_timer()
        self.ping_up = False
        # A ping timeout indicates a server connection problem. Disconnect
        # completely.
        try:
            self.client.disconnected()
        except IOError:
            # An IOError is raised by the default DisconnectHandler
            pass
        self.disconnect()
        return

    def disconnect(self):
        """ Disconnect from Jabber server. """
        if self.hook_fd != None:
            weechat.unhook(self.hook_fd)
            self.hook_fd = None
        if self.client != None:
            #if self.client.isConnected():
            #    self.client.disconnect()
            self.client = None
            self.jid = None
            self.sock = None
            self.buddy = None
            weechat.nicklist_remove_all(self.buffer)

    def close_buffer(self):
        """ Close server buffer. """
        if self.buffer != "":
            weechat.buffer_close(self.buffer)
            self.buffer = ""

    def delete(self, deleteOptions=False):
        """ Delete server. """
        for chat in self.chats:
            chat.delete()
        self.delete_ping_timer()
        self.delete_ping_timeout_timer()
        self.disconnect()
        self.close_buffer()
        if deleteOptions:
            for name, option in self.options.items():
                weechat.config_option_free(option)

def eval_expression(option_name):
    """ Return a evaluated expression """
    if int(version) >= 0x00040200:
        return weechat.string_eval_expression(option_name,{},{},{})
    else:
        return option_name

def jabber_search_server_by_name(name):
    """ Search a server by name. """
    global jabber_servers
    for server in jabber_servers:
        if server.name == name:
            return server
    return None

def jabber_search_context(buffer):
    """ Search a server / chat for a buffer. """
    global jabber_servers
    context = { "server": None, "chat": None }
    for server in jabber_servers:
        if server.buffer == buffer:
            context["server"] = server
            return context
        for chat in server.chats:
            if chat.buffer == buffer:
                context["server"] = server
                context["chat"] = chat
                return context
    return context

def jabber_search_context_by_name(server_name):
    """Search for buffer given name of server. """

    bufname = "%s.server.%s" % (SCRIPT_NAME, server_name)
    return jabber_search_context(weechat.buffer_search("python", bufname))


# =================================[ chats ]==================================

class Chat:
    """ Class to manage private chat with buddy or MUC. """

    def __init__(self, server, buddy, switch_to_buffer):
        """ Init chat """
        self.server = server
        self.buddy = buddy
        buddy.chat = self
        bufname = "%s.%s.%s" % (SCRIPT_NAME, server.name, self.buddy.alias)
        self.buffer = weechat.buffer_search("python", bufname)
        if not self.buffer:
            self.buffer = weechat.buffer_new(bufname,
                                             "jabber_buffer_input_cb", "",
                                             "jabber_buffer_close_cb", "")
        self.buffer_title = self.buddy.alias
        if self.buffer:
            weechat.buffer_set(self.buffer, "title", self.buffer_title)
            weechat.buffer_set(self.buffer, "short_name", self.buddy.alias)
            weechat.buffer_set(self.buffer, "localvar_set_type", "private")
            weechat.buffer_set(self.buffer, "localvar_set_server", server.name)
            weechat.buffer_set(self.buffer, "localvar_set_channel", self.buddy.alias)
            weechat.hook_signal_send("logger_backlog",
                                     weechat.WEECHAT_HOOK_SIGNAL_POINTER, self.buffer)
            if switch_to_buffer:
                weechat.buffer_set(self.buffer, "display", "auto")

    def recv_message(self, buddy, message):
        """ Receive a message from buddy. """
        if buddy.alias != self.buffer_title:
            self.buffer_title = buddy.alias
            weechat.buffer_set(self.buffer, "title", "%s" % self.buffer_title)
        weechat.prnt_date_tags(self.buffer, 0,
                               "notify_private,nick_%s,prefix_nick_%s,log1" %
                               (buddy.alias,
                                weechat.config_string(weechat.config_get("weechat.color.chat_nick_other"))),
                               "%s%s\t%s" % (weechat.color("chat_nick_other"),
                                             buddy.alias,
                                             message))

    def send_message(self, message):
        """ Send message to buddy. """
        if not self.server.ping_up:
            weechat.prnt(self.buffer, "%sjabber: unable to send message, connection is down"
                         % weechat.prefix("error"))
            return
        self.server.send_message(self.buddy, message)
        weechat.prnt_date_tags(self.buffer, 0,
                               "notify_none,no_highlight,nick_%s,prefix_nick_%s,log1" %
                               (self.server.buddy.alias,
                                weechat.config_string(weechat.config_get("weechat.color.chat_nick_self"))),
                               "%s%s\t%s" % (weechat.color("chat_nick_self"),
                                             self.server.buddy.alias,
                                             message))
    def print_status(self, status):
        """ Print a status message in chat. """
        weechat.prnt(self.buffer, "%s%s has status %s" %
                     (weechat.prefix("action"),
                      self.buddy.alias,
                      status))

    def close_buffer(self):
        """ Close chat buffer. """
        if self.buffer != "":
            weechat.buffer_close(self.buffer)
            self.buffer = ""

    def delete(self):
        """ Delete chat. """
        self.close_buffer()

# =================================[ buddies ]==================================

class Buddy:
    """ Class to manage buddies. """
    def __init__(self, jid=None, chat=None, server=None ):
        """ Init buddy

        Args:
            jid: xmpp.protocol.JID object instance or string
            chat: Chat object instance
            server: Server object instance

        The jid argument can be provided either as a xmpp.protocol.JID object
        instance or as a string, eg "username@domain.tld/resource". If a string
        is provided, it is converted and stored as a xmpp.protocol.JID object
        instance.
        """

        # The jid argument of xmpp.protocol.JID can be either a string or a
        # xmpp.protocol.JID object instance itself.
        self.jid = xmpp.protocol.JID(jid=jid)
        self.chat = chat
        self.server = server
        self.bare_jid = ''
        self.username = ''
        self.name = ''
        self.domain = ''
        self.resource = ''
        self.alias = ''
        self.away = True
        self.status = ''

        self.parse_jid()
        self.set_alias()
        return

    def away_string(self):
        """ Return a string with away and status, with color codes. """
        if not self:
            return ''
        if not self.away:
            return ''
        str_colon = ": "
        if not self.status:
            str_colon = ""
        return "%s(%saway%s%s%s)" % (weechat.color("chat_delimiters"),
                                      weechat.color("chat"),
                                      str_colon,
                                      self.status.replace("\n", " "),
                                      weechat.color("chat_delimiters"))

    def parse_jid(self):
        """Parse the jid property.

        The table shows how the jid is parsed and which properties are updated.

            Property        Value
            jid             myuser@mydomain.tld/myresource

            bare_jid        myuser@mydomain.tld
            username        myuser
            domain          mydomain.tld
            resource        myresource
        """
        if not self.jid:
            return
        self.bare_jid = self.jid.getStripped().encode("utf-8")
        self.username = self.jid.getNode()
        self.domain = self.jid.getDomain()
        self.resource = self.jid.getResource()
        return

    def set_alias(self):
        """Set the buddy alias.

        If an alias is defined in jabber_jid_aliases, it is used. Otherwise the
        alias is set to self.bare_jid or self.name if it exists.
        """
        self.alias = self.bare_jid
        if not self.bare_jid:
            self.alias = ''
        if self.name:
            self.alias = self.name
        global jabber_jid_aliases
        for alias, jid in jabber_jid_aliases.items():
            if jid == self.bare_jid:
                self.alias = alias
                break
        return

    def set_name(self, name=''):
        self.name = name
        self.set_alias()
        return

    def set_status(self, away=True, status=''):
        """Set the buddy status.

        Two properties define the buddy status.
            away - boolean, indicates whether the buddy is away or not.
            status - string, a message indicating the away status, eg 'in a meeting'
                   Comparable to xmpp presence <status/> element.
        """
        if not away and not status:
            status = 'online'
        # If the status has changed print a message on the server buffer
        if self.away != away or self.status != status:
            self.server.print_status(self.alias, status)
        self.away = away
        self.status = status
        return

# ================================[ commands ]================================

def jabber_hook_commands_and_completions():
    """ Hook commands and completions. """
    weechat.hook_command(SCRIPT_COMMAND, "Manage Jabber servers",
                         "list || add <name> <jid> <password> [<server>[:<port>]]"
                         " || connect|disconnect|del [<server>] || alias [add|del <alias> <jid>]"
                         " || away [<message>] || buddies || priority [<priority>]"
                         " || status [<message>] || presence [online|chat|away|xa|dnd]"
                         " || debug || set <server> <setting> [<value>]",
                         "      list: list servers and chats\n"
                         "       add: add a server\n"
                         "   connect: connect to server using password\n"
                         "disconnect: disconnect from server\n"
                         "       del: delete server\n"
                         "     alias: manage jid aliases\n"
                         "      away: set away with a message (if no message, away is unset)\n"
                         "  priority: set priority\n"
                         "    status: set status message\n"
                         "  presence: set presence status\n"
                         "   buddies: display buddies on server\n"
                         "     debug: toggle jabber debug on/off (for all servers)\n"
                         "\n"
                         "Without argument, this command lists servers and chats.\n"
                         "\n"
                         "Examples:\n"
                         "  Add a server:       /jabber add myserver user@server.tld password\n"
                         "  Add gtalk server:   /jabber add myserver user@gmail.com password talk.google.com:5223\n"
                         "  Connect to server:  /jabber connect myserver\n"
                         "  Disconnect:         /jabber disconnect myserver\n"
                         "  Delete server:      /jabber del myserver\n"
                         "\n"
                         "Aliases:\n"
                         "  List aliases:    /jabber alias \n"
                         "  Add an alias:    /jabber alias add alias_name jid\n"
                         "  Delete an alias: /jabber alias del alias_name\n"
                         "\n"
                         "Other jabber commands:\n"
                         "  Chat with a buddy (pv buffer): /jchat\n"
                         "  Add buddy to roster:           /invite\n"
                         "  Remove buddy from roster:      /kick\n"
                         "  Send message to buddy:         /jmsg",
                         "list %(jabber_servers)"
                         " || add %(jabber_servers)"
                         " || connect %(jabber_servers)"
                         " || disconnect %(jabber_servers)"
                         " || del %(jabber_servers)"
                         " || alias add|del %(jabber_jid_aliases)"
                         " || away"
                         " || priority"
                         " || status"
                         " || presence online|chat|away|xa|dnd"
                         " || buddies"
                         " || debug",
                         "jabber_cmd_jabber", "")
    weechat.hook_command("jchat", "Chat with a Jabber buddy",
                         "<buddy>",
                         "buddy: buddy id",
                         "",
                         "jabber_cmd_jchat", "")
    weechat.hook_command("jmsg", "Send a messge to a buddy",
                         "[-server <server>] <buddy> <text>",
                         "server: name of jabber server buddy is on\n"
                         " buddy: buddy id\n"
                         "  text: text to send",
                         "",
                         "jabber_cmd_jmsg", "")
    weechat.hook_command("invite", "Add a buddy to your roster",
                         "<buddy>",
                         "buddy: buddy id",
                         "",
                         "jabber_cmd_invite", "")
    weechat.hook_command("kick", "Remove a buddy from your roster, or deny auth",
                         "<buddy>",
                         "buddy: buddy id",
                         "",
                         "jabber_cmd_kick", "")
    weechat.hook_completion("jabber_servers", "list of jabber servers",
                            "jabber_completion_servers", "")
    weechat.hook_completion("jabber_jid_aliases", "list of jabber jid aliases",
                            "jabber_completion_jid_aliases", "")

def jabber_list_servers_chats(name):
    """ List servers and chats. """
    global jabber_servers
    weechat.prnt("", "")
    if len(jabber_servers) > 0:
        weechat.prnt("", "jabber servers:")
        for server in jabber_servers:
            if name == "" or server.name.find(name) >= 0:
                conn_server = ''
                if server.option_string("server"):
                    conn_server = ':'.join(
                            (server.option_string("server"),
                            server.option_string("port")))
                connected = ""
                if server.sock >= 0:
                    connected = "(connected)"

                weechat.prnt("", "  %s - %s %s %s" % (server.name,
                    eval_expression(server.option_string("jid")), conn_server, connected))
                for chat in server.chats:
                    weechat.prnt("", "    chat with %s" % (chat.buddy))
    else:
        weechat.prnt("", "jabber: no server defined")

def jabber_cmd_jabber(data, buffer, args):
    """ Command '/jabber'. """
    global jabber_servers, jabber_config_option
    if args == "" or args == "list":
        jabber_list_servers_chats("")
    else:
        argv = args.split(" ")
        argv1eol = ""
        pos = args.find(" ")
        if pos > 0:
            argv1eol = args[pos+1:]
        if argv[0] == "list":
            jabber_list_servers_chats(argv[1])
        elif argv[0] == "add":
            if len(argv) >= 4:
                server = jabber_search_server_by_name(argv[1])
                if server:
                    weechat.prnt("", "jabber: server '%s' already exists" % argv[1])
                else:
                    kwargs = {'jid': argv[2], 'password': argv[3]}
                    if len(argv) > 4:
                        conn_server, _, conn_port = argv[4].partition(':')
                        if conn_port and not conn_port.isdigit():
                            weechat.prnt("", "jabber: error, invalid port, digits only")
                            return weechat.WEECHAT_RC_OK
                        if conn_server: kwargs['server'] = conn_server
                        if conn_port: kwargs['port'] = conn_port
                    server = Server(argv[1], **kwargs)
                    jabber_servers.append(server)
                    weechat.prnt("", "jabber: server '%s' created" % argv[1])
            else:
                weechat.prnt("", "jabber: unable to add server, missing arguments")
                weechat.prnt("", "jabber: usage: /jabber add name jid password [server[:port]]")
        elif argv[0] == "alias":
            alias_command = AliasCommand(buffer, argv=argv[1:])
            alias_command.run()
        elif argv[0] == "connect":
            server = None
            if len(argv) >= 2:
                server = jabber_search_server_by_name(argv[1])
                if not server:
                    weechat.prnt("", "jabber: server '%s' not found" % argv[1])
            else:
                context = jabber_search_context(buffer)
                if context["server"]:
                    server = context["server"]
            if server:
                if weechat.config_boolean(server.options['autoreconnect']):
                    server.ping()               # This will connect and update ping status
                    server.add_ping_timer()
                else:
                    server.connect()
        elif argv[0] == "disconnect":
            server = None
            if len(argv) >= 2:
                server = jabber_search_server_by_name(argv[1])
                if not server:
                    weechat.prnt("", "jabber: server '%s' not found" % argv[1])
            else:
                context = jabber_search_context(buffer)
                if context["server"]:
                    server = context["server"]
            context = jabber_search_context(buffer)
            if server:
                server.delete_ping_timer()
                server.disconnect()
        elif argv[0] == "del":
            if len(argv) >= 2:
                server = jabber_search_server_by_name(argv[1])
                if server:
                    server.delete(deleteOptions=True)
                    jabber_servers.remove(server)
                    weechat.prnt("", "jabber: server '%s' deleted" % argv[1])
                else:
                    weechat.prnt("", "jabber: server '%s' not found" % argv[1])
        elif argv[0] == "send":
            if len(argv) >= 3:
                context = jabber_search_context(buffer)
                if context["server"]:
                    buddy = context['server'].search_buddy_list(argv[1], by='alias')
                    message = ' '.join(argv[2:])
                    context["server"].send_message(buddy, message)
        elif argv[0] == "read":
            jabber_config_read()
        elif argv[0] == "away":
            context = jabber_search_context(buffer)
            if context["server"]:
                context["server"].set_away(argv1eol)
        elif argv[0] == "priority":
            context = jabber_search_context(buffer)
            if context["server"]:
                if len(argv) == 1:
                    weechat.prnt("", "jabber: priority = %d" % int(context["server"].presence.getPriority()))
                elif len(argv) == 2 and argv[1].isdigit():
                    context["server"].set_presence(priority=int(argv[1]))
                else:
                    weechat.prnt("", "jabber: you need to specify priority as positive integer between 0 and 65535")
        elif argv[0] == "status":
            context = jabber_search_context(buffer)
            if context["server"]:
                if len(argv) == 1:
                    weechat.prnt("", "jabber: status = %s" % context["server"].presence.getStatus())
                else:
                    context["server"].set_presence(status=argv1eol)
        elif argv[0] == "presence":
            context = jabber_search_context(buffer)
            if context["server"]:
                if len(argv) == 1:
                    show =  context["server"].presence.getShow()
                    if show == "": show = "online"
                    weechat.prnt("", "jabber: presence = %s" % show)
                elif not re.match(r'^(?:online|chat|away|xa|dnd)$', argv[1]):
                    weechat.prnt("", "jabber: Presence should be one of: online, chat, away, xa, dnd")
                else:
                    if argv[1] == "online": show = ""
                    else: show = argv[1]
                    context["server"].set_presence(show=show)
        elif argv[0] == "buddies":
            context = jabber_search_context(buffer)
            if context["server"]:
                context["server"].display_buddies()
        elif argv[0] == "debug":
            weechat.config_option_set(jabber_config_option["debug"], "toggle", 1)
            if jabber_debug_enabled():
                weechat.prnt("", "jabber: debug is now ON")
            else:
                weechat.prnt("", "jabber: debug is now off")
        else:
            weechat.prnt("", "jabber: unknown action")
    return weechat.WEECHAT_RC_OK

def jabber_cmd_jchat(data, buffer, args):
    """ Command '/jchat'. """
    if args:
        context = jabber_search_context(buffer)
        if context["server"]:
            buddy = context["server"].search_buddy_list(args, by='alias')
            if not buddy:
                buddy = context["server"].add_buddy(jid=args)
            if not buddy.chat:
                context["server"].add_chat(buddy)
            weechat.buffer_set(buddy.chat.buffer, "display", "auto")
    return weechat.WEECHAT_RC_OK

def jabber_cmd_jmsg(data, buffer, args):
    """ Command '/jmsg'. """
    if args:
        argv = args.split()
        if len(argv) < 2:
            return weechat.WEECHAT_RC_OK
        if argv[0] == '-server':
            context = jabber_search_context_by_name(argv[1])
            recipient = argv[2]
            message = " ".join(argv[3:])
        else:
            context = jabber_search_context(buffer)
            recipient = argv[0]
            message = " ".join(argv[1:])
        if context["server"]:
            buddy = context['server'].search_buddy_list(recipient, by='alias')
            context["server"].send_message(buddy, message)

    return weechat.WEECHAT_RC_OK

def jabber_cmd_invite(data, buffer, args):
    """ Command '/invite'. """
    if args:
        context = jabber_search_context(buffer)
        if context["server"]:
            context["server"].add_buddy(args)
    return weechat.WEECHAT_RC_OK

def jabber_cmd_kick(data, buffer, args):
    """ Command '/kick'. """
    if args:
        context = jabber_search_context(buffer)
        if context["server"]:
            context["server"].del_buddy(args)
    return weechat.WEECHAT_RC_OK

def jabber_away_command_run_cb(data, buffer, command):
    """ Callback called when /away -all command is run """
    global jabber_servers
    words = command.split(None, 2)
    if len(words) < 2:
        return
    message = ''
    if len(words) > 2:
        message = words[2]
    for server in jabber_servers:
        server.set_away(message)
    return weechat.WEECHAT_RC_OK

class AliasCommand(object):
    """Class representing a jabber alias command, ie /jabber alias ..."""

    def __init__(self, buffer, argv=None):
        """
        Args:
            bufffer: the weechat buffer the command was run in
            argv: list, the arguments provided with the command.
                  Example, if the command is "/jabber alias add abc abc@server.tld"
                  argv = ['add', 'abc', 'abc@server.tld']
        """
        self.buffer = buffer
        self.argv = []
        if argv:
            self.argv = argv
        self.action = ''
        self.jid = ''
        self.alias = ''
        self.parse()
        return

    def add(self):
        """Run a "/jabber alias add" command"""
        global jabber_jid_aliases
        if not self.alias or not self.jid:
            weechat.prnt("", "\njabber: unable to add alias, missing arguments")
            weechat.prnt("", "jabber: usage: /jabber alias add alias_name jid")
            return
        # Restrict the character set of aliases. The characters must be writable to
        # config file.
        invalid_re = re.compile(r'[^a-zA-Z0-9\[\]\\\^_\-{|}@\.]')
        if invalid_re.search(self.alias):
            weechat.prnt("", "\njabber: invalid alias: %s" % self.alias)
            weechat.prnt("", "jabber: use only characters: a-z A-Z 0-9 [ \ ] ^ _ - { | } @ .")
            return
        # Ensure alias and jid are reasonable length.
        max_len = 64
        if len(self.alias) > max_len:
            weechat.prnt("", "\njabber: invalid alias: %s" % self.alias)
            weechat.prnt("", "jabber: must be no more than %s characters long" % max_len)
            return
        if len(self.jid) > max_len:
            weechat.prnt("", "\njabber: invalid jid: %s" % self.jid)
            weechat.prnt("", "jabber: must be no more than %s characters long" % max_len)
            return
        jid = self.jid.encode("utf-8")
        alias = self.alias.encode("utf-8")
        if alias in jabber_jid_aliases.keys():
            weechat.prnt("", "\njabber: unable to add alias: %s" % (alias))
            weechat.prnt("", "jabber: alias already exists, delete first")
            return
        if jid in jabber_jid_aliases.values():
            weechat.prnt("", "\njabber: unable to add alias: %s" % (alias))
            for a, j in jabber_jid_aliases.items():
                if j == jid:
                    weechat.prnt("", "jabber: jid '%s' is already aliased as '%s', delete first" %
                        (j, a))
                    break
        jabber_jid_aliases[alias] = jid
        self.alias_reset(jid)
        return

    def alias_reset(self, jid):
        """Reset objects related to the jid modified by an an alias command

        Update any existing buddy objects, server nicklists, and chat objects
        that may be using the buddy with the provided jid.
        """
        global jabber_servers
        for server in jabber_servers:
            buddy = server.search_buddy_list(jid, by='jid')
            if not buddy:
                continue
            server.update_nicklist(buddy=buddy, action='remove')
            buddy.set_alias()
            server.update_nicklist(buddy=buddy, action='update')
            if buddy.chat:
                switch_to_buffer = False
                if buddy.chat.buffer == self.buffer:
                    switch_to_buffer = True
                buddy.chat.delete()
                new_chat = server.add_chat(buddy)
                if switch_to_buffer:
                    weechat.buffer_set(new_chat.buffer, "display", "auto")
        return

    def delete(self):
        """Run a "/jabber alias del" command"""
        global jabber_jid_aliases
        if not self.alias:
            weechat.prnt("", "\njabber: unable to delete alias, missing arguments")
            weechat.prnt("", "jabber: usage: /jabber alias del alias_name")
            return
        if not self.alias in jabber_jid_aliases:
            weechat.prnt("", "\njabber: unable to delete alias '%s', not found" % (self.alias))
            return
        jid = jabber_jid_aliases[self.alias]
        del jabber_jid_aliases[self.alias]
        self.alias_reset(jid)
        return

    def list(self):
        """Run a "/jabber alias" command to list aliases"""
        global jabber_jid_aliases
        weechat.prnt("", "")
        if len(jabber_jid_aliases) <= 0:
            weechat.prnt("", "jabber: no aliases defined")
            return
        weechat.prnt("", "jabber jid aliases:")
        len_alias = 5
        len_jid = 5
        for alias, jid in jabber_jid_aliases.items():
            if len_alias < len(alias):
                len_alias = len(alias)
            if len_jid < len(jid):
                len_jid = len(jid)
        prnt_format = "  %-" + str(len_alias) + "s %-" + str(len_jid) + "s"
        weechat.prnt("", prnt_format % ('Alias', 'JID'))
        for alias, jid in sorted(jabber_jid_aliases.items()):
            weechat.prnt("", prnt_format % (alias, jid))
        return

    def parse(self):
        """Parse the alias command into components"""
        if len(self.argv) <= 0:
            return
        self.action = self.argv[0]
        if len(self.argv) > 1:
            # Pad argv list to prevent IndexError exceptions
            while len(self.argv) < 3: self.argv.append('')
            self.alias = self.argv[1]
            self.jid = self.argv[2]
        return

    def run(self):
        """Execute the alias command."""
        if self.action == 'add':
            self.add()
        elif self.action == 'del':
            self.delete()
        self.list()
        return

def jabber_completion_servers(data, completion_item, buffer, completion):
    """ Completion with jabber server names. """
    global jabber_servers
    for server in jabber_servers:
        weechat.hook_completion_list_add(completion, server.name,
                                         0, weechat.WEECHAT_LIST_POS_SORT)
    return weechat.WEECHAT_RC_OK

def jabber_completion_jid_aliases(data, completion_item, buffer, completion):
    """ Completion with jabber alias names. """
    global jabber_jid_aliases
    for alias, jid in sorted(jabber_jid_aliases.items()):
        weechat.hook_completion_list_add(completion, alias,
                                         0, weechat.WEECHAT_LIST_POS_SORT)
    return weechat.WEECHAT_RC_OK

# ==================================[ fd ]====================================

def jabber_fd_cb(data, fd):
    """ Callback for reading socket. """
    global jabber_servers
    for server in jabber_servers:
        if server.sock == int(fd):
            server.recv()
    return weechat.WEECHAT_RC_OK

# ================================[ buffers ]=================================

def jabber_buffer_input_cb(data, buffer, input_data):
    """ Callback called for input data on a jabber buffer. """
    context = jabber_search_context(buffer)
    if context["server"] and context["chat"]:
        context["chat"].send_message(input_data)
    elif context["server"]:
        if input_data == "buddies" or "buddies".startswith(input_data):
            context["server"].display_buddies()
        else:
            context["server"].send_message_from_input(input=input_data)
    return weechat.WEECHAT_RC_OK

def jabber_buffer_close_cb(data, buffer):
    """ Callback called when a jabber buffer is closed. """
    context = jabber_search_context(buffer)
    if context["server"] and context["chat"]:
        if context["chat"].buddy:
            context["chat"].buddy.chat = None
        context["chat"].buffer = ""
        context["server"].chats.remove(context["chat"])
    elif context["server"]:
        context["server"].buffer = ""
    return weechat.WEECHAT_RC_OK

# ==================================[ timers ]==================================

def jabber_ping_timeout_timer(server_name, remaining_calls):
    server = jabber_search_server_by_name(server_name)
    if server:
        server.ping_time_out()
    return weechat.WEECHAT_RC_OK

def jabber_ping_timer(server_name, remaining_calls):
    server = jabber_search_server_by_name(server_name)
    if server:
        server.ping()
    return weechat.WEECHAT_RC_OK

# ==================================[ main ]==================================

if __name__ == "__main__" and import_ok:
    if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION,
                        SCRIPT_LICENSE, SCRIPT_DESC,
                        "jabber_unload_script", ""):

        version = weechat.info_get("version_number", "") or 0
        jabber_hook_commands_and_completions()
        jabber_config_init()
        jabber_config_read()
        for server in jabber_servers:
            if weechat.config_boolean(server.options['autoreconnect']):
                server.ping()               # This will connect and update ping status
                server.add_ping_timer()
            else:
                if weechat.config_boolean(server.options['autoconnect']):
                    server.connect()

# ==================================[ end ]===================================

def jabber_unload_script():
    """ Function called when script is unloaded. """
    global jabber_servers
    jabber_config_write()
    for server in jabber_servers:
        server.disconnect()
        server.delete()
    return weechat.WEECHAT_RC_OK



# -*- coding: utf-8 -*-
#
# Copyright (C) 2017 p3lim <weechat@p3lim.net>
#
# https://github.com/p3lim/weechat-pushjet

try:
	import weechat
except ImportError:
	from sys import exit
	print('This script has to run under WeeChat (https://weechat.org/).')
	exit(1)

from urllib import urlencode

SCRIPT_NAME = 'pushjet'
SCRIPT_AUTHOR = 'p3lim'
SCRIPT_VERSION = '0.1.0'
SCRIPT_LICENSE = 'MIT'
SCRIPT_DESC = 'Send highlights and mentions through Pushjet.io'

SETTINGS = {
	'host': (
		'https://api.pushjet.io',
		'host for the pushjet api'),
	'secret': (
		'',
		'secret for the pushjet api'),
	'level': (
		'4',
		'severity level for the message, from 1 to 5 (low to high)'),
	'timeout': (
		'30',
		'timeout for the message sending in seconds (>= 1)'),
	'separator': (
		': ',
		'separator between nick and message in notifications'),
	'notify_on_highlight': (
		'on',
		'push notifications for highlights in buffers (on/off)'),
	'notify_on_privmsg': (
		'on',
		'push notifications for private messages (on/off)'),
	'notify_when': (
		'always',
		'when to push notifications (away/detached/always/never)'),
	'ignore_buffers': (
		'',
		'comma-separated list of buffers to ignore'),
	'ignore_nicks': (
		'',
		'comma-separated list of users to not push notifications from'),
}

def send_message(title, message):
	secret = weechat.config_get_plugin('secret')
	if secret != '':
		data = {
			'secret': secret,
			'level': int(weechat.config_get_plugin('level')),
			'title': title,
			'message': message,
		}

		host = weechat.config_get_plugin('host').rstrip('/') + '/message'
		timeout = int(weechat.config_get_plugin('timeout')) * 1000

		if timeout <= 0:
			timeout = 1

		data = urlencode(data)
		cmd = 'python -c \'from urllib2 import Request, urlopen; r = urlopen(Request("%s", "%s")); print r.getcode()\'' % (host, data)
		weechat.hook_process(cmd, timeout, 'send_message_callback', '')

def send_message_callback(data, command, return_code, out, err):
	if return_code != 0:
		# something went wrong
		return weechat.WEECHAT_RC_ERROR

	return weechat.WEECHAT_RC_OK

def get_sender(tags, prefix):
	# attempt to find sender from tags
	# nicks are always prefixed with 'nick_'
	for tag in tags:
		if tag.startswith('nick_'):
			return tag[5:]

	# fallback method to find sender from prefix
	# nicks in prefixes are prefixed with optional modes (e.g @ for operators)
	# so we have to strip away those first, if they exist
	if prefix.startswith(('~', '&', '@', '%', '+', '-', ' ')):
		return prefix[1:]

	return prefix

def get_buffer_names(buffer):
	buffer_names = []
	buffer_names.append(weechat.buffer_get_string(buffer, 'short_name'))
	buffer_names.append(weechat.buffer_get_string(buffer, 'name'))
	return buffer_names

def should_send(buffer, tags, nick, highlighted):
	if not nick:
		# a nick is required to form a correct message, bail
		return False

	if highlighted:
		if weechat.config_get_plugin('notify_on_highlight') != 'on':
			# notifying on highlights is disabled, bail
			return False
	elif weechat.buffer_get_string(buffer, 'localvar_type') == 'private':
		if weechat.config_get_plugin('notify_on_privmsg') != 'on':
			# notifying on private messages is disabled, bail
			return False
	else:
		# not a highlight or private message, bail
		return False

	notify_when = weechat.config_get_plugin('notify_when')
	if notify_when == 'never':
		# user has opted to not be notified, bail
		return False
	elif notify_when == 'away':
		# user has opted to only be notified when away
		infolist_args = (
			weechat.buffer_get_string(buffer, 'localvar_channel'),
			weechat.buffer_get_string(buffer, 'localvar_server'),
			weechat.buffer_get_string(buffer, 'localvar_nick')
		)

		if not None in infolist_args:
			infolist = weechat.infolist_get('irc_nick', '', ','.join(infolist_args))
			if infolist:
				away_status = weechat.infolist_integer(infolist, 'away')
				if not away_status:
					# user is not away, bail
					return False
	elif notify_when == 'detached':
		# user has opted to only be notified when detached (relays)
		num_relays = weechat.info_get('relay_client_count', 'connected')
		if num_relays == 0:
			# no relays connected, bail
			return False

	if nick == weechat.buffer_get_string(buffer, 'localvar_nick'):
		# the sender was the current user, bail
		return False

	if nick in weechat.config_get_plugin('ignore_nicks').split(','):
		# the sender was on the ignore list, bail
		return False

	for buffer_name in get_buffer_names(buffer):
		if buffer_name in weechat.config_get_plugin('ignore_buffers').split(','):
			# the buffer was on the ignore list, bail
			return False

	return True

def message_callback(data, buffer, date, tags, displayed, highlight, prefix, message):
	nick = get_sender(tags, prefix)

	if should_send(buffer, tags, nick, int(highlight)):
		message = '%s%s%s' % (nick, weechat.config_get_plugin('separator'), message)

		if int(highlight):
			buffer_names = get_buffer_names(buffer)
			send_message(buffer_names[0] or buffer_names[1], message)
		else:
			send_message('Private Message', message)

	return weechat.WEECHAT_RC_OK

# register plugin
weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, '', '')

# grab all messages in any buffer
weechat.hook_print('', '', '', 1, 'message_callback', '')

# register configuration defaults
for option, value in SETTINGS.items():
	if not weechat.config_is_set_plugin(option):
		weechat.config_set_plugin(option, value[0])

	weechat.config_set_desc_plugin(option, '%s (default: "%s")' % (value[1], value[0]))



# -*- coding: utf-8 -*-
# Copyright (C) 2011 godric <godric@0x3f.fr>
# License: WTFPL
#
# Changelog
# v1.3
# + fix login regex
# + completion with online contacts from contact list for send and who commands
# + prevent default completion for all other commands
# v1.2
# + PEP8 almost-compliance
# + code adjustment
# + fix bug with state_time (ValueError)
# + state in config to change it at connexion
# + added a add_contact command, to add contacts at run time
# + added a reconnect command
# + try to reconnect on server disconnection
# + reconnect_time  in config, for attempts between reconnections
# + display_name in config, for buffers base name
# v1.1
# + re-implemented state change
# + changed pattern and color in nicklist
"""A simple plugin for netsoul protocol"""

SCRIPT_NAME = 'weenetsoul'
SCRIPT_AUTHOR = 'godric <godric@0x3f.fr>'
SCRIPT_VERSION = '1.3'
SCRIPT_LICENSE = 'WTFPL'
SCRIPT_DESC = 'Netsoul protocol for WeeChat'

import weechat
import time, socket, hashlib, urllib, re
from datetime import datetime as dt
from collections import OrderedDict

########################################
# Netsoul network functions
########################################

class WeeNSUser(object):

    def __init__(self, login, data='', location='', group='', state='',
                 state_time=0, machtype='', ip='', connection_time=0,
                 lastseen_time=0, user_trust=0, client_trust=0, fd=''):
        self.login = login
        self.fd = fd
        self.data = data
        self.location = location
        self.group = group
        self.state = state
        try:
            self.state_time = (dt.fromtimestamp(int(state_time))
                               .strftime('%d/%m/%Y %H:%M:%S'))
        except:
            self.state_time = (dt.fromtimestamp(0)
                               .strftime('%d/%m/%Y %H:%M:%S'))
        self.machtype = machtype
        self.ip = ip
        try:
            self.connection_time = (dt.fromtimestamp(int(connection_time))
                                    .strftime('%d/%m/%Y %H:%M:%S'))
        except:
            self.connection_time = (dt.fromtimestamp(0)
                                    .strftime('%d/%m/%Y %H:%M:%S'))
        try:
            self.lastseen_time = (dt.fromtimestamp(int(lastseen_time))
                                  .strftime('%d/%m/%Y %H:%M:%S'))
        except:
            self.lastseen_time = (dt.fromtimestamp(0)
                                  .strftime('%d/%m/%Y %H:%M:%S'))
        self.user_trust = int(user_trust)
        self.client_trust = int(client_trust)
        self.nick = None

    def prnt(self, buffer='', prefix=''):
        hcolor = weechat.color('separator')
        ncolor = weechat.color('chat_nick')
        prefix = prefix + hcolor
        weechat.prnt(buffer,
                     ("{0}{1}{2} {3}[{4}] ({5}) {6}{2}\n"
                      .format(prefix, hcolor, '=' * 15, ncolor,
                              self.login, self.fd, hcolor)))
        weechat.prnt(buffer,
                     '%s   Host . . . . . . . . : %s\n'
                     % (prefix, self.ip))
        weechat.prnt(buffer,
                     "%s   Machine type . . . . : %s\n"
                     % (prefix, self.machtype))
        weechat.prnt(buffer,
                     "%s   Group. . . . . . . . : %s\n"
                     % (prefix, self.group))
        weechat.prnt(buffer,
                     "%s   State. . . . . . . . : %s\n"
                     % (prefix, self.state))
        weechat.prnt(buffer,
                     "%s   Data . . . . . . . . : %s\n"
                     % (prefix, self.data))
        weechat.prnt(buffer,
                     "%s   Location . . . . . . : %s\n"
                     % (prefix, self.location))
        weechat.prnt(buffer,
                     "%s   Connected at . . . . : %s\n"
                     % (prefix, self.connection_time))
        weechat.prnt(buffer,
                     "%s   Last Activity. . . . : %s\n"
                     % (prefix, self.lastseen_time))
        weechat.prnt(buffer,
                     "%s   Last status change . : %s\n"
                     % (prefix, self.state_time))
        weechat.prnt(buffer,
                     "%s   Trust (User/client). : (%d/%d)\n"
                     % (prefix, self.user_trust, self.client_trust))
        weechat.prnt(buffer,
                     "%s%s%s"
                     % (prefix, hcolor, '=' * 46))


class WeeNSChat(object):

    def __init__(self, server, login=None, fd=None):
        self.server = server
        self.login = login
        self.fd = fd
        self.buffer = weechat.buffer_new('%s.temp' % (self.server.name,),
                                         'wee_ns_buffer_input_cb', '',
                                         'wee_ns_buffer_close_cb', '')
        weechat.buffer_set(self.buffer, "display", "auto")
        self.update_buffer(login)
        self.server.chats.append(self)

    def update_buffer(self, login):
        self.login = login
        fd = self.fd or '*'
        login = self.login or 'unknown'
        weechat.buffer_set(self.buffer, 'title',
                           'Netsoul chat with %s on %s' % (login, fd))
        weechat.buffer_set(self.buffer, 'name',
                           '%s.%s:%s' % (self.server.name, login, fd))
        weechat.buffer_set(self.buffer, 'short_name',
                           '%s:%s' % (login, fd))
        weechat.buffer_set(self.buffer, 'nick',
                           '%s' % (self.server.get_option('login'),))
        weechat.buffer_set(self.buffer, 'channel',
                           '%s:%s' % (login, fd))
        weechat.buffer_set(self.buffer, 'server',
                           '%s' % (self.server.get_option('host'),))
        weechat.buffer_set(self.buffer, 'type',
                           'private')

    def recv(self, login, message):
        if login != self.login:
            self.update_buffer(login)
        weechat.prnt(self.buffer,
                     '%s%s\t%s'
                     % (weechat.color("chat_nick"), self.login, message))

    def send(self, message):
        weechat.prnt(self.buffer,
                     '%s%s\t%s'
                     % (weechat.color("chat_nick_self"),
                        self.server.get_option('login'), message))
        recipient = self.login if self.fd is None else (':' + self.fd)
        self.server._ns_user_cmd_msg_user(recipient, message)

    def delete(self):
        if self.buffer is not None:
            return weechat.buffer_close(self.buffer)
        self.server.chats.remove(self)


class WeeNSServer(object):

    def __init__(self):
        self.buffer = None
        self.hook_fd = None
        self.socket = None
        self.netbuffer = ''
        self.chats = []
        self.contacts = {}
        lastfields = (0, '', '', '', '', '', '')
        self.options = {
            'host': weechat.config_new_option(wee_ns_config_file,
                                              wee_ns_conf_serv_sect,
                                              'host', 'string',
                                              'Server host', '', 0, 0,
                                              'ns-server.epita.fr',
                                              'ns-server.epita.fr',
                                              *lastfields),
            'port': weechat.config_new_option(wee_ns_config_file,
                                              wee_ns_conf_serv_sect,
                                              'port', 'string',
                                              'Server port', '', 0, 0,
                                              '4242', '4242',
                                              *lastfields),
            'login': weechat.config_new_option(wee_ns_config_file,
                                               wee_ns_conf_serv_sect,
                                               'login', 'string',
                                               'User login (ie: login_x)',
                                               '', 0, 0, 'login_x', 'login_x',
                                               *lastfields),
            'password': weechat.config_new_option(wee_ns_config_file,
                                                  wee_ns_conf_serv_sect,
                                                  'password', 'string',
                                                  ('User password (ie: '
                                                   'your SOCKS password)'),
                                                  '', 0, 0, 'xxxxxx', 'xxxxxx',
                                                  *lastfields),
            'location': weechat.config_new_option(wee_ns_config_file,
                                                  wee_ns_conf_serv_sect,
                                                  'location', 'string',
                                                  ('User location (ie: '
                                                   'at home)'),
                                                  '', 0, 0, '-', '-',
                                                  *lastfields),
            'data': weechat.config_new_option(wee_ns_config_file,
                                              wee_ns_conf_serv_sect,
                                              'data', 'string',
                                              ('User data (ie: '
                                               'j\'aime les chips)'),
                                              '', 0, 0, '-', '-',
                                              *lastfields),
            'state': weechat.config_new_option(wee_ns_config_file,
                                               wee_ns_conf_serv_sect,
                                               'state', 'string',
                                               'User state (ie: actif)',
                                               '', 0, 0, 'actif', 'actif',
                                               *lastfields),
            'contacts': weechat.config_new_option(wee_ns_config_file,
                                                  wee_ns_conf_serv_sect,
                                                  'contacts', 'string',
                                                  ('Comma separated login '
                                                   'list (ie: sb,rn)'),
                                                  '', 0, 0, '', '',
                                                  *lastfields),
            'display_name': weechat.config_new_option(wee_ns_config_file,
                                                      wee_ns_conf_serv_sect,
                                                      'display_name', 'string',
                                                      ('Server display name '
                                                       'in buffer list '
                                                       '("" = host value)'),
                                                      '', 0, 0,
                                                      'Netsoul', 'Netsoul',
                                                      *lastfields),
            'reconnect_time': weechat.config_new_option(wee_ns_config_file,
                                                        wee_ns_conf_serv_sect,
                                                        'reconnect_time',
                                                        'string',
                                                        ('Time in seconds '
                                                         'between two '
                                                         'reconnection '
                                                         'attempts '
                                                         '(0 = no '
                                                         'reconnection)'),
                                                        '', 0, 0, '30', '30',
                                                        *lastfields),
        }
        self.commands_hooks = {
            'salut': self._ns_parse_salut,
            'ping': self._ns_parse_ping,
            'user_cmd': self._ns_parse_user_cmd,
        }
        self.user_cmd_hooks = {
            'msg': self._ns_parse_user_cmd_msg,
            'who': self._ns_parse_user_cmd_who,
            'login': self._ns_parse_user_cmd_login,
            'logout': self._ns_parse_user_cmd_logout,
            'state': self._ns_parse_user_cmd_state,
        }

    @property
    def name(self):
        return {
            True: self.get_option('display_name'),
            False: self.get_option('host'),
        }[self.get_option('display_name') != ""]

    def get_option(self, opt_name):
        return weechat.config_string(self.options[opt_name])

    def get_default(self, opt_name):
        return weechat.config_default_string(self.options[opt_name])

    def get_chat_by_recipient(self, login=None, fd=None, create=False):
        for chat in self.chats:
            if ((fd is not None and chat.fd == fd) or
                (chat.login == login and chat.fd == fd)):
                return chat
        if create is True:
            return WeeNSChat(self, login, fd)
        return None

    def get_chat_by_buffer(self, buffer):
        for chat in self.chats:
            if chat.buffer == buffer:
                return chat
        return server

    def create_buffer(self):
        if self.buffer == None:
            self.buffer = weechat.buffer_new('server.%s' % (self.name,),
                                             'wee_ns_buffer_input_cb', '',
                                             'wee_ns_buffer_close_cb', '')
            weechat.buffer_set(self.buffer, "nicklist",
                               "1")
            weechat.buffer_set(self.buffer, "nicklist_display_groups",
                               "1")
            weechat.buffer_set(self.buffer, "display",
                               "auto")
            weechat.buffer_set(self.buffer, "nick",
                               "%s" % (self.get_option('login'),))
            weechat.buffer_set(self.buffer, "server",
                               "%s" % (self.get_option('host'),))
            weechat.buffer_set(self.buffer, "channel",
                               "%s" % (self.name,))
            weechat.buffer_set(self.buffer, "type",
                               "server")

    def connect(self):
        self.create_buffer()
        self.disconnect()
        weechat.prnt(self.buffer,
                     'Connecting to %s:%s'
                     % (self.get_option('host'), self.get_option('port')))
        try:
            hints = socket.getaddrinfo(self.get_option('host'),
                                       self.get_option('port'),
                                       socket.AF_UNSPEC, socket.SOCK_STREAM,
                                       0, socket.AI_PASSIVE)
        except socket.error as e:
            weechat.prnt(self.buffer,
                         '%s%s' % (weechat.prefix('error'), e.args[1]))
        else:
            for res in hints:
                af, socktype, proto, canonname, sa = res
                try:
                    self.socket = socket.socket(af, socktype, proto)
                    self.socket.connect(sa)
                except socket.error:
                    self.socket = None
                    continue
                break
        if self.socket is None:
            weechat.prnt(self.buffer,
                         '%sCould not connect' % (weechat.prefix('error'),))
        else:
            self.hook_fd = weechat.hook_fd(self.socket.fileno(), 1, 0, 0,
                                           'wee_ns_hook_fd_cb', '')

    def recv(self):
        data = self.socket.recv(512)
        if len(data) == 0:
            return self.reconnect()
        self.netbuffer += data
        while "\n" in self.netbuffer:
            line, ignored, buffer = self.netbuffer.partition("\n")
            index = len(line) + 1
            self.netbuffer = self.netbuffer[index:]
            weechat.prnt(self.buffer,
                         '%s[%s]' % (weechat.prefix('join'), line))
            self._ns_parse(line)

    def send(self, data):
        weechat.prnt(self.buffer,
                     '%s[%s]' % (weechat.prefix('quit'), data))
        self.socket.send('%s\n' % data)

    @property
    def is_connected(self):
        return self.socket is not None

    def disconnect(self):
        if self.hook_fd is not None:
            weechat.unhook(self.hook_fd)
        if self.socket is not None:
            self.socket.close()
        if self.buffer is not None:
            self.contacts = {}
            weechat.nicklist_remove_all(self.buffer)
            weechat.prnt(self.buffer, '... Disconnected ...')
        self.hook_fd = None
        self.socket = None
        self.contacts = {}

    def reconnect(self):
        if self.is_connected:
            self.disconnect()
        wee_ns_reconnect_loop()

    def delete(self):
        for chat in reversed(self.chats):
            chat.delete()
        self.disconnect()
        if self.buffer is not None:
            weechat.buffer_close(self.buffer)
            self.buffer = None
        for option in self.options:
            weechat.config_option_free(option)

    def setup_nick_list(self):
        contact_list = {
            True: [],
            False: self.get_option('contacts').replace(' ', '').split(',')
        }[self.get_option('contacts').replace(' ', '') == '']
        for login in contact_list:
            weechat.nicklist_add_group(self.buffer, '', login, 'lightcyan', 1)
            self.contacts[login] = {}
        self._ns_user_cmd_who('{%s}' % ','.join(contact_list))
        self._ns_user_cmd_watch_log_user(','.join(contact_list))

    def add_to_nick_list(self, *logins):
        contact_list = {
            True: [],
            False: self.get_option('contacts').replace(' ', '').split(',')
        }[self.get_option('contacts').replace(' ', '') == '']
        new_contacts = []
        for login in logins:
            if login in self.contacts:
                continue
            contact_list.append(login)
            new_contacts.append(login)
            self.contacts[login] = {}
            weechat.nicklist_add_group(self.buffer, '', login, 'lightcyan', 1)
        self._ns_user_cmd_who('{%s}' % ','.join(new_contacts))
        self._ns_user_cmd_watch_log_user(','.join(contact_list))
        weechat.config_option_set(self.options['contacts'],
                                  ','.join(contact_list), 0)

    def update_nick_list(self, user, remove=False):
        group = weechat.nicklist_search_group(self.buffer, '', user.login)
        if group is not None and user.login in self.contacts:
            if user.fd in self.contacts[user.login]:
                weechat.nicklist_remove_nick(self.buffer,
                                             (self.contacts[user.login]
                                              [user.fd].nick))
                del self.contacts[user.login][user.fd]
            if remove is False:
                user.nick = weechat.nicklist_add_nick(self.buffer, group,
                                                      ' :%s%s@%s%s'
                                                      % (user.fd,
                                                         (weechat
                                                          .color('separator')),
                                                         (weechat
                                                          .color('default')),
                                                         user.location),
                                                      '', '', '', 1)
                self.contacts[user.login][user.fd] = user

    def _ns_auth_ag(self):
        self.send("auth_ag ext_user none none")

    def _ns_ext_user_log(self, secret, ip, port):
        location = urllib.quote(self.get_option('location'))
        data = urllib.quote(self.get_option('data'))
        crypt = hashlib.md5('%s-%s/%s%s'
                            % (secret, ip, port,
                               self.get_option('password'))).hexdigest()
        self.send("ext_user_log %s %s %s %s" % (self.get_option('login'),
                                                crypt, location, data))

    def _ns_user_cmd_msg_user(self, login, msg):
        msg = unicode(msg, 'utf-8').encode('iso-8859-1')
        self.send("user_cmd msg_user %s msg %s" % (login, urllib.quote(msg)))

    def _ns_user_cmd_who(self, login):
        self.send("user_cmd who %s" % login)

    def _ns_user_cmd_watch_log_user(self, friends):
        self.send("user_cmd watch_log_user {%s}" % friends)

    def _ns_state(self, state):
        self.send("state %s:%s" % (state, int(time.time())))

    def _ns_parse(self, data):
        arglist = data.split(' ')
        if arglist[0] in self.commands_hooks:
            self.commands_hooks[arglist[0]](arglist)

    def _ns_parse_from(self, str):
        r = re.compile(r"""
        ([0-9]+):                # fd
        user:
        ([0-9]+)/([0-9]+):       # user_trust
        ([_a-z0-9-]+)@([0-9.]+): # login@ip
        ([^ :]+):                # machtype
        ([^ :]+):                # location
        ([^ :]+)                 # group
        """, re.VERBOSE)
        match = re.match(r, str)
        groups = match.groups()
        user = WeeNSUser(fd=groups[0], client_trust=groups[1],
                         user_trust=groups[2], login=groups[3],
                         ip=groups[4], machtype=groups[5],
                         location=groups[6], group=groups[7])
        return user

    def _ns_parse_salut(self, arglist):
        self._ns_auth_ag()
        self._ns_ext_user_log(arglist[2], arglist[3], arglist[4])
        self._ns_state(self.get_option('state'))
        self.setup_nick_list()

    def _ns_parse_ping(self, arglist):
        self.send("ping %s" % arglist[1])

    def _ns_parse_user_cmd(self, arglist):
        if arglist[3] in self.user_cmd_hooks:
            self.user_cmd_hooks[arglist[3]](arglist)

    def _ns_parse_user_cmd_msg(self, arglist):
        user = self._ns_parse_from(arglist[1])
        msg = urllib.unquote(arglist[4])
        msg = unicode(msg, 'iso-8859-1').encode('utf-8')
        pchat = self.get_chat_by_recipient(fd=user.fd)
        if pchat is not None:
            pchat.recv(user.login, msg)
        gchat = self.get_chat_by_recipient(user.login, create=(pchat is None))
        if gchat is not None:
            gchat.recv(user.login, msg)

    def _ns_parse_user_cmd_who(self, arglist):
        r = re.compile(r"""
        who[ ]
        ([0-9]+)[ ]             # fd
        ([_a-z0-9-]+)[ ]        # login
        ([0-9.]+)[ ]            # ip
        ([0-9]+)[ ]             # connection_time
        ([0-9]+)[ ]             # lastseen_time
        ([0-9]+)[ ]             # user_trust
        ([0-9]+)[ ]             # client_trust
        ([^ ]+)[ ]              # machtype
        ([^ ]+)[ ]              # location
        ([^ ]+)[ ]              # group
        ([^ :]+)(:([^ ]+)?)?[ ] # state:state_time
        ([^ ]+)                 # data
        """, re.VERBOSE)
        match = re.match(r, ' '.join(arglist[3:]))
        if match is not None:
            groups = match.groups()
            user = WeeNSUser(fd=groups[0], login=groups[1], ip=groups[2],
                             connection_time=groups[3],
                             lastseen_time=groups[4],
                             user_trust=groups[5], client_trust=groups[6],
                             machtype=groups[7],
                             location=urllib.unquote(groups[8]),
                             group=groups[9],
                             state=urllib.unquote(groups[10]),
                             state_time=groups[12] or 0,
                             data=urllib.unquote(groups[13]))
            self.update_nick_list(user)
            user.prnt(self.buffer, weechat.prefix('network'))

    def _ns_parse_user_cmd_state(self, arglist):
        user = self._ns_parse_from(arglist[1])
        if (user.login in self.contacts and
            user.fd in self.contacts[user.login]):
            user = self.contacts[user.login][user.fd]
            state = arglist[4].split(':')
            user.state = urllib.unquote(state[0])
            user.state_time = (dt.fromtimestamp(int(time.time()))
                               .strftime('%d/%m/%Y %H:%M:%S'))
            self.update_nick_list(user)
            msg = '%s%s%s changed state to %s' % (weechat.prefix('network'),
                                                  weechat.color('nick_color'),
                                                  user.login, user.state)
            pchat = self.get_chat_by_recipient(fd=user.fd)
            if pchat is not None:
                weechat.prnt(pchat.buffer, msg)
            gchat = self.get_chat_by_recipient(user.login)
            if gchat is not None:
                weechat.prnt(gchat.buffer, msg)

    def _ns_parse_user_cmd_login(self, arglist):
        user = self._ns_parse_from(arglist[1])
        self._ns_user_cmd_who(':%s' % user.fd)
        msg = '%s%s%s has connected' % (weechat.prefix('join'),
                                        weechat.color('nick_color'),
                                        user.login)
        pchat = self.get_chat_by_recipient(fd=user.fd)
        if pchat is not None:
            weechat.prnt(pchat.buffer, msg)
        gchat = self.get_chat_by_recipient(user.login)
        if gchat is not None:
            weechat.prnt(gchat.buffer, msg)

    def _ns_parse_user_cmd_logout(self, arglist):
        user = self._ns_parse_from(arglist[1])
        self.update_nick_list(user, remove=True)
        msg = '%s%s%s has disconnected' % (weechat.prefix('quit'),
                                           weechat.color('nick_color'),
                                           user.login)
        pchat = self.get_chat_by_recipient(fd=user.fd)
        if pchat is not None:
            weechat.prnt(pchat.buffer, msg)
        gchat = self.get_chat_by_recipient(user.login)
        if gchat is not None:
            weechat.prnt(gchat.buffer, msg)


########################################
# Weechat callbacks
########################################

def wee_ns_reconnect_loop(*args):
    weechat.prnt(server.buffer, 'Trying to reconnect...')
    server.connect()
    try:
        reconnect_time = int(server.get_option('reconnect_time'))
    except ValueError:
        reconnect_time = int(server.get_default('reconnect_time'))
    if not server.is_connected and reconnect_time > 0:
        weechat.prnt(server.buffer,
                     'Failed, next attempt in %s seconds'
                     % (server.get_option('reconnect_time'),))
        weechat.hook_timer(reconnect_time * 1000, 0, 1,
                           'wee_ns_reconnect_loop', '')
    return weechat.WEECHAT_RC_OK

def wee_ns_buffer_input_cb(data, buffer, input_data):
    if server.is_connected:
        server.get_chat_by_buffer(buffer).send(input_data)
    return weechat.WEECHAT_RC_OK

def wee_ns_buffer_close_cb(data, buffer):
    context = server.get_chat_by_buffer(buffer)
    context.buffer = None
    context.delete()
    return weechat.WEECHAT_RC_OK

def wee_ns_serv_sect_read_cb(data, config_file, section, option_name, value):
    return weechat.config_option_set(server.options[option_name], value, 0)

def wee_ns_hook_fd_cb(data, fd):
    server.recv()
    return weechat.WEECHAT_RC_OK

def wee_ns_script_unload_cb():
    weechat.config_write(wee_ns_config_file)
    return weechat.WEECHAT_RC_OK

def wee_ns_hook_cmd_connect(server, *args):
    """Connect you to netsoul"""
    if not server.is_connected:
        server.connect()
    else:
        return "Already connected"

def wee_ns_hook_cmd_disconnect(server, *args):
    """Disconnect you from netsoul"""
    if server.is_connected:
        server.disconnect()
    else:
        return "Not connected"

def wee_ns_hook_cmd_reconnect(server, *args):
    """Reconnect you to netsoul"""
    server.reconnect()

def wee_ns_hook_cmd_send(server, *args):
    """Send <msg> to <login> (any client) or <:fd> (unique client)"""
    if len(args) < 2:
        return -1
    if not server.is_connected:
        return "Not connected"
    match = re.match(re.compile(r'''
    \A
    ([_a-z0-9-]+) # login
    |:([0-9]+)    # or :fd
    \Z
    ''', re.VERBOSE), args[0])
    if not match:
        return "Message recipients must be of type <login> or <:fd>"
    server.get_chat_by_recipient(*match.groups(),
                                 create=True).send(' '.join(args[1:]))

def wee_ns_hook_cmd_state(server, *args):
    """Change status to <status> (en ligne/actif/whatever)"""
    if len(args) < 1:
        return -1
    if not server.is_connected:
        return "Not connected"
    server._ns_state(args[0])

def wee_ns_hook_cmd_who(server, *args):
    """Show infos about all <login>... specified"""
    if len(args) < 1:
        return -1
    if not server.is_connected:
        return "Not connected"
    server._ns_user_cmd_who('{%s}' % ','.join(args))

def wee_ns_hook_cmd_add_contact(server, *args):
    """Add all <login>... specified to your contacts"""
    if len(args) < 1:
        return -1
    if not server.is_connected:
        return "Not connected"
    server.add_to_nick_list(*args)

def wee_ns_hook_completion_send(data, completion_item,
                                buffer, completion):
    if server.is_connected:
        [weechat.hook_completion_list_add(completion, contact, 0,
                                          weechat.WEECHAT_LIST_POS_SORT)
         for contact, fds in server.contacts.items() if fds]
    return weechat.WEECHAT_RC_OK

def wee_ns_hook_cmd_ns(data, buffer, args):
    buffer = server.buffer if server.buffer is not None else ''
    arglist = args.split(' ')
    if arglist[0] in hook_cmd_ns:
        target = hook_cmd_ns[arglist[0]]
        res = target['cb'](server, *arglist[1:])
        if res is not None:
            if res == -1:
                res = 'usage: %s%s' % (arglist[0],
                                       '' if 'desc' not in target
                                       else " " + target['desc'])
            weechat.prnt(buffer, '%s%s' % (weechat.prefix('error'), res))
    else:
        weechat.prnt(buffer,
                     ('%sNo such command, '
                      'wrong argument count, '
                      'or you need to (dis)connect') % weechat.prefix('error'))
    return weechat.WEECHAT_RC_OK

# Add your commands here
hook_cmd_ns = OrderedDict([
    ('connect', {'cb': wee_ns_hook_cmd_connect,
                 'compl': '%-'}),
    ('disconnect', {'cb': wee_ns_hook_cmd_disconnect,
                    'compl': '%-'}),
    ('reconnect', {'cb': wee_ns_hook_cmd_reconnect,
                   'compl': '%-'}),
    ('send', {'cb': wee_ns_hook_cmd_send,
              'desc': "<login> <msg>",
              'compl': '%(ns_send) %-'}),
    ('state', {'cb': wee_ns_hook_cmd_state,
               'desc': "<status>",
               'compl': '%-'}),
    ('who', {'cb': wee_ns_hook_cmd_who,
             'desc': "<login>...",
             'compl': '%(ns_send) %-'}),
    ('add_contact', {'cb': wee_ns_hook_cmd_add_contact,
                     'desc': "<login>...",
                     'compl': '%-'}),
])

######################################
# Main
######################################

if __name__ == "__main__":
    if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION,
                        SCRIPT_LICENSE, SCRIPT_DESC, '',
                        'wee_ns_script_unload_cb'):
        weechat.hook_completion('ns_send', 'login completion',
                                'wee_ns_hook_completion_send', '')
        weechat.hook_command('ns', 'weeNetsoul: A netsoul plugin for weechat',
                             ' | '.join("%s%s" % (k, "" if 'desc' not in v
                                                  else " " + v['desc'])
                                        for k, v in hook_cmd_ns.items()),
                             '\n'.join("%s: %s" % (k, v['cb'].__doc__)
                                       for k, v in hook_cmd_ns.items()),
                             ' || '.join("%s%s" % (k, "" if 'compl' not in v
                                                   else " " + v['compl'])
                                         for k, v in hook_cmd_ns.items()),
                             'wee_ns_hook_cmd_ns', '')
        wee_ns_config_file = weechat.config_new(SCRIPT_NAME, '', '')
        wee_ns_conf_serv_sect = weechat.config_new_section(wee_ns_config_file,
                                                           'server', 0, 0,
                                                           ('wee_ns_serv_'
                                                            'sect_read_cb'),
                                                           '', '', '', '', '',
                                                           '', '', '', '')
        server = WeeNSServer()
        weechat.config_read(wee_ns_config_file)
        weechat.config_write(wee_ns_config_file)



"""
    mpv.py

    author: teraflops <cprieto.ortiz@gmail.com>
    contributor: nashgul <m.alcocer1978@gmail.com>
      desc: Sends your current mpv track to the current buffer
     usage:
       /mpv
    note: you have to run this way: mpv --input-ipc-server=/tmp/mpvsocket

   license: The Beer-ware License

"""

import weechat
import subprocess
from json import loads

MPV = {
    'SCRIPT_NAME'     : 'mpv',
    'SCRIPT_COMMAND'  : 'mpv',
    'default_options' : {
                            'message'    : '/me is now watching: ',
                            'mpv_socket' : '/tmp/mpvsocket'
                        },
}

def set_default_options():
    for option, default_value in MPV['default_options'].items():
        if not weechat.config_is_set_plugin(option):
            weechat.config_set_plugin(option, default_value)

def reload_options_cb(data, option, value):
    MPV[option.split('.')[-1]] = value
    return weechat.WEECHAT_RC_OK

def load_options():
    for option in MPV['default_options'].keys():
        MPV[option] = weechat.config_get_plugin(option)

def mpv_msg(world,world_eol,userdata):
    try:
        output = subprocess.check_output("echo \'{ \"command\": [\"get_property\", \"metadata\"] }\' | socat - %s" % MPV['mpv_socket'], shell=True)
        output_short = subprocess.check_output("echo \'{ \"command\": [\"get_property\", \"media-title\"] }\' | socat - %s" % MPV['mpv_socket'], shell=True)
        metadata = loads(output.decode('utf8'))
        metadata_short = loads(output_short.decode('utf8'))
        if 'album' not in metadata['data']:
            title = metadata_short['data'].encode('utf-8')
            all = '%s' % MPV['message'] + title
            weechat.command(weechat.current_buffer(), all)
            return weechat.WEECHAT_RC_OK

        if 'album' in metadata['data']:
            title = metadata['data']['title'].encode('utf-8')
            artist = metadata['data']['artist'].encode('utf-8')
            np = artist + ' ' + title
            all = '%s' %  MPV['message'] + np
            weechat.command(weechat.current_buffer(), all)
            return weechat.WEECHAT_RC_OK
        else:
            return weechat.WEECHAT_RC_ERROR
    except:
        weechat.prnt('', '%s: mpv socket not properly configurated or mpv is not running' % MPV['SCRIPT_NAME'])
        return weechat.WEECHAT_RC_ERROR

weechat.register(MPV['SCRIPT_NAME'], "llua", "0.1", "The Beer-ware License", "Now Playing for mpv", "", "")
set_default_options()
load_options()
weechat.hook_config('plugins.var.python.' + MPV['SCRIPT_NAME'] + '.*', 'reload_options_cb', '')
weechat.hook_command(MPV['SCRIPT_COMMAND'], "Now Watching", "", "/%s" % MPV['SCRIPT_COMMAND'], "", "mpv_msg", "")



# -*- coding: utf-8 -*-
###
# Copyright (c) 2010 by xt <xt@bash.no>
# License: GPL3
#
#
#   History:
#   2010-11-08
#   version 0.1: initial release
#
###

SCRIPT_NAME    = "tailer"
SCRIPT_AUTHOR  = "xt <xt@bash.no>"
SCRIPT_VERSION = "0.1"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "Tail any number of files and run any command with line appended"

### Default Settings ###
settings = {
        'entries': '/var/log/logfile=/mute msg -server freenode #flood', # filename and command
        'interval': '10', # in seconds
}


entries = {}
hook = ''
try:
    import weechat
    w = weechat
    WEECHAT_RC_OK = weechat.WEECHAT_RC_OK
    import_ok = True
except:
    print "This script must be run under WeeChat."
    print "Get WeeChat now at: http://www.weechat.org/"
    import_ok = False

import os

def tailer_conf_update(*args):
    global hook
    interval = int(w.config_get_plugin('interval'))
    # Don't hook default setting
    if not w.config_get_plugin('entries') == settings['entries']:
        if hook:
            w.unhook(hook)
        hook = w.hook_timer(interval*1000, 0, 0, 'tailer_cb', '')
    return WEECHAT_RC_OK


def tailer_cb(*args):
    for entry in w.config_get_plugin('entries').split(','):
        if not entry: continue
        filename, command = entry.split('=')
        if not filename in entries:
            #Find the size of the file and move to the end
            st_results = os.stat(filename)
            st_size = st_results[6]
            entries[filename] = st_size

        position = entries[filename]

        t_file = file(filename,'r')
        t_file.seek(position)
        lines = t_file.readlines()
        if not command.startswith('/'):
            w.prnt('', '%s: Error: %s' %(SCRIPT_NAME, 'command must start with /'))
            return WEECHAT_RC_OK
        for line in lines:
            if line:
                w.command('', '%s %s' %(command, line))
        # Update new position in file
        entries[filename] = t_file.tell()
    return WEECHAT_RC_OK

if __name__ == '__main__' and import_ok and \
        weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC,
        '', ''):


    for opt, val in settings.iteritems():
        if not weechat.config_is_set_plugin(opt):
            weechat.config_set_plugin(opt, val)

    weechat.hook_config('plugins.var.python.%s' %SCRIPT_NAME, 'tailer_conf_update', '')
    tailer_conf_update() # To init hook


# vim:set shiftwidth=4 tabstop=4 softtabstop=4 expandtab textwidth=100:



# TinyUrl, version 3.8, for weechat version 0.3.0 or later
#
#   Listens to all channels for long URLs, and submits them to ln-s.net or
#   tinyurl.com for easier links.
#
# Usage:
#
#   By default, any URL longer than 30 characters in length is grabbed,
#   submitted to a service, and printed in the channel for your eyes only.  For
#   example, you may see something like this:
#
# [11:21] <@lack> http://www.cbc.ca/story/canada/national/2005/11/12/mcdona
#                 lds-051112.html?ref=rss 
# [11:21] -P- [AKA] http://tinyurl.com/9dthl
#
#   Now you can just cut&paste the easier, shorter URL into your favourite
#   browser.
# 
#   If you want to be extra-helpful (or annoying) to certain channels you
#   are in, you can actually have the script say the tinyurl.com equivalent
#   of all long URLs, by adding the channel to the 'activechans' list.  In
#   that case, everyone in the channel would see the following:
#
# [11:25] <testuser> http://www.cbc.ca/story/canada/national/2005/11/12/mcdona
#                    lds-051112.html?ref=rss 
# [11:25] <@lack> [AKA] http://tinyurl.com/9dthl
#
# Configuration:
#
#   Run '/help tinyurl' for the actual usage for setting these options:
#
#   activechans
#     A comma-delimited list of channels you will actually "say" the
#     tinyurl in.  By default the list is empty.  Be warned, some channels
#     won't appreciate extra help (or 'noise' as they like to call it), and
#     some channels already have bots that do this.  Please only enable
#     this in channels where the ops have given you permission.
#
#   urllength
#     An integer, default value 30.  Any URL this long or longer will
#     trigger a tinyurl event.
#
#   printall
#     Either "on" or "off", default "on".  When ON, you will see the
#     tinyurl printed in your window for any channels not in your
#     activechans list.  When OFF, you will not see any tinyurls except in
#     your activechans list.
#
#   service
#     Either "tinyurl" or "ln-s".  tinyurl is the default.
#
# Requirements:
#
#  - Designed to run with weechat version 0.3.0 or better.
#      http://weechat.flashtux.org/
#
#  - Requires that 'curl' is in the path (tested with curl 7.15.0).
#      http://curl.haxx.se/
#
# Copyright (C) 2005 Jim Ramsay <i.am@jimramsay.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
# USA.
#
# Changelog:
#
# Version 3.8, 12 December, 2009
#   Update WeeChat site
#   by FlashCode <flashcode@flashtux.org>
#
# Version 3.7, 2 May, 2009
#   Sync with last API changes
#   by FlashCode <flashcode@flashtux.org>
#
# Version 3.6, 12 March, 2009
#   Conversion to WeeChat 0.3.0+
#   by FlashCode <flashcode@flashtux.org>
#
# Version 3.5d, 19 September, 2008
#   Updated with recent api changes
#   by Chris Hills <chaz@chaz6.com>
#
# Version 3.5c, 18 July, 2008
#   Send the url to the correct channel in activechans
#   by Chris Hills <chaz@chaz6.com>
#
# Version 3.5b, 18 July, 2008
#   Fixed a few prnt functional calls
#   by Chris Hills <chaz@chaz6.com>
#
# Version 3.5a, 18 July, 2008
#   Updated for the new script api in weechat 0.3.0-dev
#   by Chris Hills <chaz@chaz6.com>
#   Updated to use the tinyurl api instead of screen-scraping
#   by Chris Hills <chaz@chaz6.com>
#
# Version 3.5, June 3, 2008
#   Added ln-s.net support
#   Turned off some ugly verbose output unless requested
#
# Version 3.4, June 3, 2008
#   tinyurl.com changed their output page slightly, and broke our parsing
#
# Version 3.3, July 4, 2006
#   Catches possible error in os.waitpid
#   Properly prints tinyurls in query windows
#
# Version 3.2, June 15, 2006
#   Multiple configuration bugfixes, pointed out by Stalwart on #weechat.
#
# Version 3.1, June 15, 2006
#   Now kills any leftover curl processes when the script is unloaded.
#   Thanks again to kolter for the great idea!
#   Also cleaned up /tinyurl command, added comletion_template, updated
#   help text, improved option parsing logic, etc.
#
# Version 3.0, June 15, 2006
#   Fixes "tinyurl script sometimes makes weechat freeze" issue by using
#   the new timer handlers available in Weechat 0.1.9
#   Also includes URL detection fix from Raimund Specht
#   <raimund@spemaus.de>.
#
# Version 2.0, Dec 13, 2005
#   Also catches https, ftp, and ftps URLs, thanks to kolter for the
#   suggestion!
#
# Version 1.1, Dec 2, 2005
#   Fixed undefined 'urlend' thanks to kolter@irc.freenode.org#weechat
#
# TODO:
#
# - Handle outgoing messages and replace long urls with the tinyurl
#   equivalent automatically.
# - On load, check that 'curl' is installed, and fail if not.
#

import os, tempfile, re
try:
	import urllib
except:
	raise ImportError("You need to reload the python plugin to reload urllib")
import weechat

class TryAgain(UserWarning):
	def __init__(self, message):
		super(UserWarning, self).__init__(message)

# Register with weechat
weechat.register( "TinyUrl", "Jim Ramsay", "3.8", "GPL", "Waits for URLs and sends them to 'tinyurl' for you", "tinyurlShutdown", "" )

# Global variables
tinyurlParams = ("urllength","activechans","printall","service","debug")
tinyurlProcessList = {}

# Set default settings values:
if weechat.config_get_plugin('urllength') == "":
	weechat.config_set_plugin('urllength', "30")
if not weechat.config_get_plugin('printall') in ("on", "off"):
	weechat.config_set_plugin('printall', "on")
if not weechat.config_get_plugin('service') in ("tinyurl", "ln-s"):
	weechat.config_set_plugin('service', "tinyurl")
if not weechat.config_get_plugin('debug') in ("on", "off"):
	weechat.config_set_plugin('debug', "off")

# Start the timer thread and register handlers
weechat.hook_timer( 1000, 0, 0, "tinyurlCheckComplete", "" )
weechat.hook_signal("*,irc_in_privmsg", "tinyurlHandleMessage", "")
weechat.hook_command("tinyurl", "Sets/Gets 'tinyurl' settings.", "urllength|activechans|printall|service|debug", \
	"[<variable> [[=] <value>]]",
"""When run without arguments, displays all tinyurl settings

<variable> : Sets or displays a single tinyurl setting. One of:
    activechans [[=] #chan1[,#chan2...]]
        List of channels where others will see your tinyurls.
        Default: None
    urllength [[=] length]
        Will not create tinyurls for any URLs shorter than this.
        Default: 30
    printall [[=] on|off]
        When off, will not display private tinyurls, just those
        displayed publicly in your "active channels"
        Default: on
    service [[=] tinyurl|ln-s]
        Sets the service used for shortening the URL.
        Default: tinyurl
    debug [[=] on|off]
        Creates extra debug output in the server window when on.
        Default: off""",
	"tinyurlMain", ""
    )

def tinyurlShutdown():
	"""Cleanup - Kills any leftover child processes"""
	if len(tinyurlProcessList.keys()) > 0:
		weechat.prnt( weechat.buffer_search("",""), "-TinyUrl- Cleaning up unfinished processes:" )
		for pid in tinyurlProcessList.keys():
			weechat.prnt( weechat.buffer_search("",""), "	Process %d" % pid )
			try:
				os.kill(pid, 9)
				os.waitpid( pid, 0 )
			except:
				weechat.prnt( weechat.buffer_search("",""), "		Cleanup failed, skipping" )
	return weechat.WEECHAT_RC_OK
	
def tinyurlGet( name = "" ):
	"""Gets a variable value"""
	if name == "":
		weechat.prnt( weechat.buffer_search("",""), "-TinyUrl- Get all:" )
		for name in tinyurlParams:
			weechat.prnt( weechat.buffer_search("",""), "	%s = %s" % (name, weechat.config_get_plugin(name)) )
	else:
		weechat.prnt( weechat.buffer_search("",""), "-TinyUrl- Get:" )
		if name in tinyurlParams:
			weechat.prnt( weechat.buffer_search("",""), "	%s = %s" % (name, weechat.config_get_plugin(name)) )
		else:
			weechat.prnt( weechat.buffer_search("",""), "	Unknown parameter \"%s\", try '/help tinyurl'" % name )
	return

def tinyurlSet( name, value ):
	"""Sets a variable value"""
	if value == "":
		tinyurlGet( name )
	else:
		weechat.prnt( weechat.buffer_search("",""), "-TinyUrl- Set:" )
		if name in tinyurlParams:
			if name == "printall" or name == "debug":
				if value == "0" or value.lower() == "no" or value.lower() == "off":
					value = "off"
				elif value == "1" or value.lower() == "yes" or value.lower() == "on":
					value = "on"
				else:
					weechat.prnt( weechat.buffer_search("",""), "	%s must be one of 'on' or 'off'" % name )
					weechat.prnt( weechat.buffer_search("",""), "	value = '%s'" % value )
					return
			elif name == "service":
				if value.lower() in ("tinyurl", "ln-s"):
					value = value.lower()
				else:
					weechat.prnt( weechat.buffer_search("",""), "	service must be one of 'tinyurl' or 'ln-s'" )
					weechat.prnt( weechat.buffer_search("",""), "	value = '%s'" % value )
					return
			elif name == "urllength":
				try:
					v = int(value)
					if v < 0 or v > 100:
						weechat.prnt( weechat.buffer_search("",""), "	urllength must be between 0 and 100" )
						weechat.prnt( weechat.buffer_search("",""), "	value = '%s'" % value )
						return
				except:
					weechat.prnt( weechat.buffer_search("",""), "	urllength must be a valid integer" )
					weechat.prnt( weechat.buffer_search("",""), "	value = '%s'" % value )
					return
			elif name == "activechans":
				vs = re.split(", |,| ", value)
				values = []
				for v in vs:
					if v.startswith("#"):
						values.append(v)
						value = ",".join(values)
			weechat.config_set_plugin(name, value)
			weechat.prnt( weechat.buffer_search("",""), "	%s = %s" % (name, weechat.config_get_plugin(name)) )
		else:
			weechat.prnt( weechat.buffer_search("",""), "	Unknown parameter \'%s\'" % name )
	return

def tinyurlMain( data, buffer, args ):
	"""Main handler for the /tinyurl command"""
	args = args.split( " " )
	while '' in args:
		args.remove('')
	while ' ' in args:
		args.remove(' ')
	if len(args) == 0:
		tinyurlGet()
	else:
		name = args[0]
		value = ""
		if len(args) > 1:
			if args[1] == "=":
				value = " ".join(args[2:])
			else:
				value = " ".join(args[1:])
			tinyurlSet( args[0], value )
		else:
			tinyurlGet( name )
	return weechat.WEECHAT_RC_OK

def tinyurlGetUrl( url, channel, server ):
	"""Starts a background process which will query the appropriate service and
	put the result in a file that the timer function 'tinyurlCheck' will find and
	parse."""
	global tinyurlProcessList
	handle, filename = tempfile.mkstemp( prefix="weechat-tinyurl.py-" )
	os.close(handle)
	service = weechat.config_get_plugin('service')
	if service == "tinyurl":
		cmd = ("curl -d url=%s http://tinyurl.com/api-create.php --stderr /dev/null -o %s" % \
			 (urllib.quote(url), filename)).split()
	else:
		cmd = ("curl http://ln-s.net/home/api.jsp?url=%s --stderr /dev/null -o %s" % \
			 (urllib.quote(url), filename)).split()
	try:
		pid = os.spawnvp( os.P_NOWAIT, cmd[0], cmd )
		if weechat.config_get_plugin('debug') == "on":
			weechat.prnt( weechat.buffer_search(server,""), "Setting ProcessList[%d] to (%s, %s, %s)" % \
				(pid, filename, channel, server))
		tinyurlProcessList[pid] = (filename, url, service, channel, server)
	except Exception, e:
		weechat.prnt( weechat.buffer_search("",""), "-TinyUrl- Error: Could not spawn curl: %s" % (e) )

def parseTinyurl( file ):
	turl = None
	for line in file:
		if( line.startswith("http://tinyurl.com") ):
			turl = line
			break
	if turl is None:
		weechat.prnt( weechat.buffer_search("",""), "-TinyUrl- Error: Unrecognized response from server" )
		weechat.prnt( weechat.buffer_search("",""), "          Maybe tinyurl.com changed their format again." )
		weechat.prnt( weechat.buffer_search("",""), "          Try '/tinyurl service ln-s' to use ln-s.net instead" )
	return turl

def parseLns( file ):
	turl = None
	for line in file:
		(code, message) = line.split(" ", 2)
		if code == "200":
			return message.rstrip()
		elif code == "503":
			# Try again, respawn curl
			weechat.prnt( weechat.buffer_search("",""), "-TinyUrl- Warning: ln-s.net is busy, trying again shortly" )
			raise TryAgain(line)
		else:
			weechat.prnt( weechat.buffer_search("",""), "-TinyUrl- Error: Error response from server: %s" % (line) )
			return None

def tinyurlParsefile( filename, service ):
	"""Parses the given HTML file and pulls out the tinyurl."""
	turl = None
	try:
		html = open(filename, "r")
		if service == "tinyurl":
			turl = parseTinyurl(html)
		else:
			turl = parseLns(html)
		html.close()
	except Exception, e:
		weechat.prnt( weechat.buffer_search("",""), "-TinyUrl- Error: Could not open result file %s: %s" % (filename, e) )
	return turl

def tinyurlPrint( original, url, channel, server ):
	"""Prints the new tinyurl either to just you, or to the whole channel"""
	where = "Unknown"
	locstart = original.find("//")
	if locstart > -1:
		locend = original.find("/", locstart + 2)
		if locend > -1:
			where = original[locstart + 2:locend]
	activeChans = weechat.config_get_plugin('activechans').split(',')
	if channel in activeChans:
		weechat.command( weechat.buffer_search("irc",server + "." + channel), "/msg %s [AKA] %s" % (channel, url) )
	else:
		if weechat.config_get_plugin('debug') == "on":
			weechat.prnt( weechat.buffer_search("irc",server + "." + channel), "Printing url to channel '%s', server '%s'" % \
				(channel, server) )
		weechat.prnt( weechat.buffer_search("irc",server + "." + channel), "[AKA] %s (%s)" % (url, where) )

def tinyurlFindUrlstart( msg, start = 0 ):
	"""Finds the beginnings of URLs"""
	index = -1
	if start < 0 or start >= len(msg):
		return index
	for prefix in ( "http://", "https://", "ftp://", "ftps://" ):
		index = msg.find( prefix, start )
		if index > -1:
			break
	return index 

def tinyurlFindUrlend( msg, urlstart ):
	"""Finds the ends of URLs (Strips following punctuation)"""
	m = msg[urlstart:]
	index = m.find( " " )
	if index == -1:
		index = len(m)
	while msg[index-1] in ( "?", ".", "!" ):
		index -= 1
	return index + urlstart

def tinyurlCheckComplete(data, remaining_calls):
	"""The periodic poll of all waiting processes"""
	global tinyurlProcessList
	for pid in tinyurlProcessList.keys():
		(filename, url, service, channel, server) = tinyurlProcessList[pid]
		try:
			(p, er) = os.waitpid( pid, os.WNOHANG )
			if p != 0:
				if er == 0:
					try:
						tinyurl = tinyurlParsefile(filename, service)
						if tinyurl is not None:
							tinyurlPrint( url, tinyurl, channel, server )
					except TryAgain:
						tinyurlGetUrl(url, channel, server)
				else:
					weechat.prnt( weechat.buffer_search("",""), "-TinyUrl- Error: 'curl' did not run properly" )
				os.unlink(filename)
				del tinyurlProcessList[pid]
		except OSError, e:
			weechat.prnt( weechat.buffer_search("",""), "-TinyUrl- Error: 'curl' process not found: %s" % (e) )
			os.unlink(filename)
			del tinyurlProcessList[pid]
	return weechat.WEECHAT_RC_OK

def tinyurlHandleMessage( data, signal, signal_data ):
	"""Handles IRC PRIVMSG and checks for URLs"""
	(server,sig) = signal.split(",",1)
	maxlen = int(weechat.config_get_plugin( "urllength" ))
	activeChans = weechat.config_get_plugin('activechans').split(',')
	onlyActiveChans = weechat.config_get_plugin('printall') == "off"
	(source, type, channel, msg) = signal_data.split(" ", 3)
	if onlyActiveChans and channel not in activeChans:
		return weechat.WEECHAT_RC_OK
	if not channel.startswith("#"):
		channel = source.split("!", 2)[0][1:]
	urlstart = tinyurlFindUrlstart( msg )
	while urlstart > -1 and urlstart is not None:
		urlend = tinyurlFindUrlend( msg, urlstart )
		url = msg[urlstart:urlend]
		if len(url) >= maxlen:
			tinyurlGetUrl(url, channel, server)
		# Check for more URLs
		urlstart = tinyurlFindUrlstart( msg, urlend+1 )
	return weechat.WEECHAT_RC_OK



# ~*~ coding: utf-8 ~*~
# Author: Frantisek Kolacek <work@kolacek.it>
# Homepage: https://github.com/fkolacek/weechat-telnot

import weechat

try:
    # Python 3
    from urllib.parse import urlencode
except ImportError:
    from urllib import urlencode

weechat.register('telnot',
                 'Frantisek Kolacek <work@kolacek.it>',
                 '1.0',
                 'MIT',
                 'telnot: Send notification over Telegram using TelNot',
                 '',
                 '')

settings = {
    'endpoint': 'Address of server running TelNot instance (including http:// or https://)',
    'token': 'User token',
    'bot': 'Name of Telegram bot',
}

required_settings = [
    'endpoint',
    'token',
    'bot',
]

for option, description in list(settings.items()):
    if not weechat.config_is_set_plugin(option):
        weechat.config_set_plugin(option, '')

    if option in required_settings and weechat.config_get_plugin(option) == '':
        weechat.prnt('', weechat.prefix('error') + 'telnot: Please set option: {}'.format(option))
        weechat.prnt('', 'telnot: /set plugins.var.python.telnot.{} STRING'.format(option))

    weechat.config_set_desc_plugin(option, description)

# buffer, tags, message, strip_colors, callback, callback_data
weechat.hook_print('', 'notify_message', '', 1, 'process_notification', '')
weechat.hook_print('', 'notify_private', '', 1, 'process_notification', '')


def process_notification(data, buffer, date, tags, displayed, highlight, prefix, message):

    if 'notify_message' in tags and not highlight:
        return weechat.WEECHAT_RC_OK

    nick = weechat.buffer_get_string(buffer, 'localvar_nick')
    name = weechat.buffer_get_string(buffer, 'name')
    server = weechat.buffer_get_string(buffer, 'localvar_server')
    channel = weechat.buffer_get_string(buffer, 'localvar_channel')

    if weechat.buffer_get_string(buffer, 'localvar_type') == 'private' and prefix != nick:
        send_notification(server, channel, prefix, message)
    elif int(highlight):
        buff = weechat.buffer_get_string(buffer, 'short_name') or name

        send_notification(server, buff, prefix, message)

    return weechat.WEECHAT_RC_OK


def send_notification(server, channel, nick, message):
    endpoint = weechat.config_get_plugin('endpoint')
    token = weechat.config_get_plugin('token')
    bot = weechat.config_get_plugin('bot')

    if channel == nick:
        output = '[{}@{}] {}'.format(nick, server, message)
    else:
        output = '[{}@{}] {}: {}'.format(nick, server, channel, message)

    data = urlencode({
        'message': output,
        'token': token,
        'bot': bot,
    })

    options = {
        'postfields': data,
        'ssl_verifypeer': '0',
        'ssl_verifyhost': '0',
    }

    weechat.hook_process_hashtable('url:' + endpoint, options, 2000, '', '')



# Copyright 2018 adtac <weechat@adtac.in>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of
# this software and associated documentation files (the "Software"), to deal in
# the Software without restriction, including without limitation the rights to
# use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
# the Software, and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
# FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
# COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
# IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

# This script allows you to enable push notifications for WeeChat messages on
# your devices using Pushover. See the GitHub repo for more details:
#   https://github.com/adtac/weepushover

import json
import re
import urllib
import time

import weechat as w


description = 'push notifications from weechat to pushover'


help_text = '''
settings: (prefix with plugins.var.python.weepushover.<setting>)
    token                 your application token (required)
    user                  your user token (required)
    ignored_channels      space-separated list of channels to ignore
    subscribed_channels   space-separated list of channels to subscribe to
    away_only             send only when marked as away
    inactive_only         send only when the message is in an inactive buffer
    min_notify_interval   minimum number of seconds to wait before another notification
    debug                 print debug messages in core
'''


configs = {                      # some sane defaults
    'token': '_required',
    'user': '_required',
    'ignored_channels': '',      # no ignored channels
    'subscribed_channels': '',   # no subscribed channels
    'away_only': '1',            # send only when away
    'inactive_only': '0',        # send even if buffer is active
    'min_notify_interval': '60', # send notifications at least a minute apart
    'debug': '0',                # enable debugging
}


last_notification = 0


def debug(msg):
    if str(w.config_get_plugin('debug')) != '0':
        w.prnt('', '[weepushover] debug: {}'.format(str(msg)))


def register():
    global last_notification
    last_notification = 0

    w.register('weepushover', 'adtac', '0.1', 'MIT', description, '', '')


def load_settings():
    for (option, default_value) in configs.items():
        if w.config_get_plugin(option) == '':
            if configs[option] == '_required':
                w.prnt('', 'missing plugins.var.python.weepushover.{}'.format(option))
            else:
                w.config_set_plugin(option, configs[option])


def setup_hooks():
    global description
    global help_text

    w.hook_print('', '', '', 1, 'message_hook', '')
    w.hook_command('weepushover', description, '[command]', help_text, '', 'show_help', '')


def get_channels(kind):
    channels = w.config_get_plugin('{}_channels'.format(kind)).strip()
    if channels == '':
        return set([])
    else:
        return set([channel.strip() for channel in channels.split(' ') if channel])


def show_help(data, buffer, args):
    w.prnt('', help_text)
    return w.WEECHAT_RC_OK


def away_only_check(bufferp):
    if w.config_get_plugin('away_only') != '1':
        return False

    return not w.buffer_get_string(bufferp, 'localvar_away')


def inactive_only_check(bufferp):
    if w.config_get_plugin('inactive_only') != '1':
        return False

    return w.current_buffer() == bufferp


def interval_limit_check():
    interval = w.config_get_plugin('min_notify_interval')

    if interval is None or interval == '':
        return False

    try:
        interval = int(interval)
    except ValueError:
        w.prnt('', '[weepushover] min_notify_interval not an integer')
        return False

    global last_notification

    debug('current={}, last_notification={}, earliest_possible={}'.format(
        time.time(), last_notification, last_notification + interval))
    return time.time() < last_notification + interval


def get_buf_name(bufferp):
    short_name = w.buffer_get_string(bufferp, 'short_name')
    name = w.buffer_get_string(bufferp, 'name')
    return (short_name or name).decode('utf-8')


def is_ignored(bufferp):
    buf_name = get_buf_name(bufferp)
    return buf_name in get_channels('ignored')


def is_subscribed(bufferp):
    buf_name = get_buf_name(bufferp)
    return buf_name in get_channels('subscribed')


def message_hook(data, bufferp, uber_empty, tagsn, is_displayed, is_highlighted, prefix, message):
    is_pm = w.buffer_get_string(bufferp, 'localvar_type') == 'private'
    regular_channel = not is_subscribed(bufferp) and not is_pm

    if away_only_check(bufferp):
        debug('failed away_only_check; skipping')
        return w.WEECHAT_RC_OK

    if inactive_only_check(bufferp):
        debug('failed inactive_only_check; skipping')
        return w.WEECHAT_RC_OK

    if interval_limit_check():
        debug('failed interval_limit_check; skipping')
        return w.WEECHAT_RC_OK

    if is_ignored(bufferp) and regular_channel:
        debug('ignored regular channel skipping')
        return w.WEECHAT_RC_OK

    if not is_displayed:
        debug('not a displayed message; skipping')
        return w.WEECHAT_RC_OK

    if not is_highlighted and regular_channel:
        debug('not highlighted on a regular channel; skipping')
        return w.WEECHAT_RC_OK

    debug('passed all checks')

    if is_pm:
        title = 'Private message from {}'.format(prefix.decode('utf-8'))
    else:
        title = 'Message on {} from {}'.format(get_buf_name(bufferp), prefix.decode('utf-8'))

    send_push(title=title, message=message.decode('utf-8'))

    global last_notification
    last_notification = time.time()

    return w.WEECHAT_RC_OK


def http_request_callback(data, url, status, response, err):
    j = json.loads(response)
    if j['status'] != '1':
        w.prnt('', '[weepushover] error: {}'.format(response))
        return w.WEECHAT_RC_ERROR

    return w.WEECHAT_RC_OK


def send_push(title, message):
    postfields = {
        'token': w.config_get_plugin('token'),
        'user': w.config_get_plugin('user'),
        'title': title,
        'message': message,
        'url': 'https://glowing-bear.org',
    }

    w.hook_process_hashtable(
        'url:https://api.pushover.net/1/messages.json',
        {'postfields': urllib.urlencode(postfields)},
        20*1000,
        'http_request_callback',
        '',
    )


def main():
    register()
    load_settings()
    setup_hooks()


main()



# -*- coding: utf-8 -*-
# ex:sw=4 ts=4:ai:
#
# Copyright (c) 2012 by Krister Svanlund <krister.svanlund@gmail.com>
#   based on tcl version:
#    Remote Notification Script v1.1
#    by Gotisch <gotisch@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# Example usage when Weechat is running on a remote PC and you want
# want to use port 4321 for the connection.
#
#     On the "client" (where the notifications will end up), host is
#     the remote host where weechat is running:
#		python2 location/of/pyrnotify.py 4321 & ssh -R 4321:localhost:4321 username@host
#     You can have a second argument to specified the time to display the notification
#       python2 location/of/pyrnotify.py 4321 2000 & ssh -R 4321:localhost:4321 username@host
#     Important to remember is that you should probably setup the
#     connection with public key encryption and use something like
#     autossh to do this in the background.
#
#     In weechat:
#		/python load pyrnotify.py
#		and set the port
#		/set plugins.var.python.pyrnotify.port 4321
#
# It is also possible to set which host pyrnotify shall connect to,
# this is not recommended. Using a ssh port-forward is much safer
# and doesn't require any ports but ssh to be open.

# ChangeLog:
#
# 2018-08-20: Make it work with python3
#              use of sendall instead of send
# 2014-05-10: Change hook_print callback argument type of displayed/highlight
#             (WeeChat >= 1.0)
# 2012-06-19: Added simple escaping to the title and body strings for
#             the script to handle trailing backslashes.

from __future__ import print_function

try:
    import weechat as w
    in_weechat = True
except ImportError as e:
    in_weechat = False

import os, sys, re
import socket
import subprocess
import shlex

SCRIPT_NAME    = "pyrnotify"
SCRIPT_AUTHOR  = "Krister Svanlund <krister.svanlund@gmail.com>"
SCRIPT_VERSION = "1.1"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "Send remote notifications over SSH"

def escape(s):
    return re.sub(r'([\\"\'])', r'\\\1', s)

def run_notify(icon, nick, chan, message):
    host = w.config_get_plugin('host')
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.connect((host, int(w.config_get_plugin('port'))))
        msg = "normal {0} \"{1} to {2}\" \"{3}\"".format(icon, nick, escape(chan), escape(message))
        s.sendall(msg.encode('utf-8'))
        s.close()
    except Exception as e:
        w.prnt("", "Could not send notification: {0}".format(e))

def on_msg(*a):
    if len(a) == 8:
        data, buffer, timestamp, tags, displayed, highlight, sender, message = a
        if data == "private" or int(highlight):
            if data == "private" and w.config_get_plugin('pm-icon'):
                icon = w.config_get_plugin('pm-icon')
            else:
                icon = w.config_get_plugin('icon')
            buffer = "me" if data == "private" else w.buffer_get_string(buffer, "short_name")
            run_notify(icon, sender, buffer, message)
            #w.prnt("", str(a))
    return w.WEECHAT_RC_OK

def weechat_script():
    settings = {'host' : "localhost",
                'port' : "4321",
                'icon' : "utilities-terminal",
                'pm-icon' : "emblem-favorite"}
    if w.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, "", ""):
        for (kw, v) in settings.items():
            if not w.config_get_plugin(kw):
                w.config_set_plugin(kw, v)
        w.hook_print("", "notify_message", "", 1, "on_msg", "")
        w.hook_print("", "notify_private", "", 1, "on_msg", "private")
        w.hook_print("", "notify_highlight", "", 1, "on_msg", "") # Not sure if this is needed


######################################
## This is where the client starts, except for the global if-check nothing below this line is
## supposed to be executed in weechat, instead it runs when the script is executed from
## commandline.

def accept_connections(s, timeout=None):
    conn, addr = s.accept()
    try:
        data = ""
        d = conn.recv(1024)
        while d:
            data += d.decode('utf-8')
            d = conn.recv(1024)
    finally:
        conn.close()
    if data:
        try:
            urgency, icon, title, body = shlex.split(data)
            if timeout:
                subprocess.call(["notify-send", "-t", timeout, "-u", urgency, "-c", "IRC", "-i", icon, escape(title), escape(body)])
            else:
                subprocess.call(["notify-send", "-u", urgency, "-c", "IRC", "-i", icon, escape(title), escape(body)])

        except ValueError as e:
            print(e)
        except OSError as e:
            print(e)
    accept_connections(s, timeout)

def weechat_client(argv):
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    s.bind(("localhost", int(argv[1] if len(sys.argv) > 1 else 4321)))
    s.listen(5)
    try:
        accept_connections(s, argv[2] if len(sys.argv) > 2 else None)
    except KeyboardInterrupt as e:
        print("Keyboard interrupt")
        print(e)
    finally:
        s.close()

if __name__ == '__main__':
    if in_weechat:
        weechat_script()
    else:
        weechat_client(sys.argv)



# -*- coding: utf-8 -*-
"""
Author: Gosuke Miyashita <gosukenator@gmail.com>
Homepage: https://github.com/mizzy/weechat-plugins/
Version: 1.3
License: MIT License

This plugin is for pushing notifications to im.kayac.com.
See: http://im.kayac.com/

This plugin is based on notifo_notify.py.
See: http://www.weechat.org/scripts/source/stable/notifo_notify.py/

Original license is:

Author: SAEKI Yoshiyasu <laclef_yoshiyasu@yahoo.co.jp>
Homepage: http://bitbucket.org/laclefyoshi/weechat/
Version: 1.0
License: MIT License

This plugin requires "notifo" in your iPod touch/iPhone
See here: http://notifo.com/
"""

import weechat
import urllib
import hashlib

## registration

weechat.register("im_kayac_com_notify", "Gosuke Miyashita", "1.3", "MIT License",
    "im_kayac_com_notify: Push notification to iPod touch/iPhone with im.kayac.com", "", "")

## settings

script_options = {
    "username":  "",
    "password":  "",
    "secretkey": "",
}

if weechat.config_get_plugin("username") == "":
    weechat.prnt("", weechat.prefix("error") + "im_kayac_com_notify: Please set option: username")
    weechat.prnt("", "im_kayac_com_notify: /set plugins.var.python.im_kayac_com_notify.username STRING")

if weechat.config_get_plugin("password") == "" and weechat.config_get_plugin("secretkey") == "":
    weechat.prnt("", weechat.prefix("error") + "im_kayac_com_notify: Please set option: password or secretkey")
    weechat.prnt("", "im_kayac_com_notify: /set plugins.var.python.im_kayac_com_notify.[password|secretkey] STRING")

## functions

def postIm(message, handler=None, label=None, title=None, buffer_name=None, prefix=None):
    USERNAME  = weechat.config_get_plugin("username")
    PASSWORD  = weechat.config_get_plugin("password")
    SECRETKEY = weechat.config_get_plugin("secretkey")

    if USERNAME != "":
        url = "http://im.kayac.com/api/post/" + USERNAME
        opt_dict = {
            "message": "[%s][%s] - %s\n%s %s" % (label, buffer_name, title, prefix, message),
            }

        if PASSWORD != "":
            opt_dict["password"] = PASSWORD
        elif SECRETKEY != "":
            opt_dict["sig"] = hashlib.sha1(opt_dict["message"] + SECRETKEY).hexdigest()
        else:
            return

        opt = urllib.urlencode(opt_dict)
        cmd = "python -c 'from urllib2 import Request, urlopen; urlopen(Request(\"%s\", \"%s\"))'" % (url, opt)
        weechat.hook_process(cmd, 10000, "hook_process_cb", "")

def hook_process_cb(data, command, rc, stdout, stderr):
    return weechat.WEECHAT_RC_OK

def print_callback(data, buffer, date, tags, displayed, highlight, prefix, message):
    buffer_name = weechat.buffer_get_string(buffer, "name")
    if int(highlight):
        postIm(message, label="weechat", title="Highlight", buffer_name=buffer_name, prefix=prefix)
    elif "notify_private" in tags.split(','):
        postIm(message, label="weechat", title="Private Message", buffer_name=buffer_name, prefix=prefix)
    return weechat.WEECHAT_RC_OK

weechat.hook_print("", "", "", 1, "print_callback", "");




# -*- coding: utf-8 -*-
###
# Copyright (c) 2010 by xt <xt@bash.no>
# Parts from inotify.py by Elián Hanisch <lambdae2@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###
#   * plugins.var.python.away_action.ignore_channel:
#   Comma separated list of patterns for define ignores. Notifications from channels where its name
#   matches any of these patterns will be ignored.
#   Wildcards '*', '?' and char groups [..] can be used.
#   An ignore exception can be added by prefixing '!' in the pattern.
#
#       Example:
#       *ubuntu*,!#ubuntu-offtopic
#       any notifications from a 'ubuntu' channel will be ignored, except from #ubuntu-offtopic
#
#   * plugins.var.python.away_action.ignore_nick:
#   Same as ignore_channel, but for nicknames.
#
#       Example:
#       troll,b[0o]t
#       will ignore notifications from troll, bot and b0t
#
#   * plugins.var.python.away_action.ignore_text:
#   Same as ignore_channel, but for the contents of the message.

###
#
#
#   History:
#   2019-10-02:
#   version 0.6: make compatible with python 3
#   2014-05-10:
#   version 0.5: change hook_print callback argument type of
#                displayed/highlight (WeeChat >= 1.0)
#   2013-05-18:
#   version 0.4: add include_channel option - contributed by Atluxity
#   2010-11-04:
#   version 0.3: minor cleanups, fix import, add hook info
#   2010-03-17:
#   version 0.2: add force on option
#   2010-03-11
#   version 0.1: initial release
#
###

from __future__ import print_function

SCRIPT_NAME    = "away_action"
SCRIPT_AUTHOR  = "xt <xt@bash.no>"
SCRIPT_VERSION = "0.6"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "Run command on highlight and privmsg when away"

### Default Settings ###
settings = {
'ignore_channel' : '',
'ignore_nick'    : '',
'ignore_text'    : '',
'command'        : '/mute msg ', # Command to be ran, nick and message will be inserted at the end
'force_enabled'  : 'off',
'include_channel': 'off', # Option to include channel in insert after command.
}

ignore_nick, ignore_text, ignore_channel = (), (), ()
last_buffer = ''
try:
    import weechat
    w = weechat
    WEECHAT_RC_OK = weechat.WEECHAT_RC_OK
    import_ok = True
    from fnmatch import fnmatch
except:
    print("This script must be run under WeeChat.")
    print("Get WeeChat now at: http://www.weechat.org/")
    import_ok = False

class Ignores(object):
    def __init__(self, ignore_type):
        self.ignore_type = ignore_type
        self.ignores = []
        self.exceptions = []
        self._get_ignores()

    def _get_ignores(self):
        assert self.ignore_type is not None
        ignores = weechat.config_get_plugin(self.ignore_type).split(',')
        ignores = [ s.lower() for s in ignores if s ]
        self.ignores = [ s for s in ignores if s[0] != '!' ]
        self.exceptions = [ s[1:] for s in ignores if s[0] == '!' ]

    def __contains__(self, s):
        s = s.lower()
        for p in self.ignores:
            if fnmatch(s, p):
                for e in self.exceptions:
                    if fnmatch(s, e):
                        return False
                return True
        return False

config_string = lambda s : weechat.config_string(weechat.config_get(s))
def get_nick(s):
    """Strip nickmodes and prefix, suffix."""
    if not s: return ''
    # prefix and suffix
    prefix = config_string('irc.look.nick_prefix')
    suffix = config_string('irc.look.nick_suffix')
    if s[0] == prefix:
        s = s[1:]
    if s[-1] == suffix:
        s = s[:-1]
    # nick mode
    modes = '~+@!%'
    s = s.lstrip(modes)
    return s

def away_cb(data, buffer, time, tags, display, hilight, prefix, msg):

    global ignore_nick, ignore_text, ignore_channel, last_buffer

    # Check if we are either away or force_enabled is on
    if not w.buffer_get_string(buffer, 'localvar_away') and \
       not w.config_get_plugin('force_enabled') == 'on':
        return WEECHAT_RC_OK

    if (int(hilight) or 'notify_private' in tags) and int(display):
        channel = weechat.buffer_get_string(buffer, 'short_name')
        prefix = get_nick(prefix)
        if prefix not in ignore_nick \
                and channel not in ignore_channel \
                and msg not in ignore_text:
            last_buffer = w.buffer_get_string(buffer, 'plugin') + '.' + \
                          w.buffer_get_string(buffer, 'name')
            command = weechat.config_get_plugin('command')
            if not command.startswith('/'):
                w.prnt('', '%s: Error: %s' %(SCRIPT_NAME, 'command must start with /'))
                return WEECHAT_RC_OK

            if 'channel' in locals() and \
                w.config_get_plugin('include_channel') == 'on':
                w.command('', '%s @%s <%s> %s' %(command, channel, prefix, msg))
            else:
                w.command('', '%s <%s> %s' %(command, prefix, msg))
    return WEECHAT_RC_OK

def ignore_update(*args):
    ignore_channel._get_ignores()
    ignore_nick._get_ignores()
    ignore_text._get_ignores()
    return WEECHAT_RC_OK


def info_hook_cb(data, info_name, arguments):
    global last_buffer
    return last_buffer


if __name__ == '__main__' and import_ok and \
        weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC,
        '', ''):


    for opt, val in settings.items():
        if not weechat.config_is_set_plugin(opt):
            weechat.config_set_plugin(opt, val)

    ignore_channel = Ignores('ignore_channel')
    ignore_nick = Ignores('ignore_nick')
    ignore_text = Ignores('ignore_text')
    weechat.hook_config('plugins.var.python.%s.ignore_*' %SCRIPT_NAME, 'ignore_update', '')

    weechat.hook_print('', '', '', 1, 'away_cb', '')
    w.hook_info('%s_buffer' %SCRIPT_NAME, '', '', 'info_hook_cb', '')


# vim:set shiftwidth=4 tabstop=4 softtabstop=4 expandtab textwidth=100:
